#ifndef __GUILLOTINE_H__
#define __GUILLOTINE_H__

#define __DEBUG_GUILLOTINE__				1

/* src/guillotine */

typedef struct s_guillotine_stat
{

  unsigned long			memory;
  int				nb;

} guillotine_stat_t;

typedef struct s_guillotine_terminal
{

 
} guillotine_terminal_t;

enum
  {

    ENV_TYPE_PUBLIC,
    ENV_TYPE_PRIVATE

  } guillotine_env_e_type;

typedef struct s_guillotine_env
{

  char						name[SANSON_SIZE_STRING_NAME];
  char						value[SANSON_SIZE_STRING_NAME];
  
  int						type;

  TAILQ_ENTRY(s_guillotine_env)			entries;
  
} guillotine_env_t;


TAILQ_HEAD(s_tailq_env, s_guillotine_env);

typedef struct s_guillotine_session
{
  
  guillotine_terminal_t				terminal;
  guillotine_stat_t				stats;

  sanson_session_t				sst;

  char						*name;

} guillotine_session_t;


typedef struct s_guillotine_get
{

  char						name[SANSON_SIZE_STRING_NAME];	
  int						(*func)(char *);

} guillotine_get_t;

typedef struct s_guillotine_cmd
{

  char						name[SANSON_SIZE_CMD];
  int						(*func)(guillotine_session_t *, char *);

} guillotine_cmd_t;


#define GUILLOTINE_PROMPT			"guillotine# "

/* main.c */


/* judgment.c */

int wait_judgment(guillotine_session_t *);
int guillotine_cmd_send(guillotine_session_t *, char *);
int guillotine_cmd_del(guillotine_session_t *, char *);
int guillotine_cmd_exec(guillotine_session_t *, char *);
int guillotine_cmd_list(guillotine_session_t *, char *);
int guillotine_cmd_option(guillotine_session_t *, char *);
int guillotine_cmd_exit(guillotine_session_t *, char *);
int guillotine_cmd_kmb(guillotine_session_t *, char *);

/* exec.c */

int pre_build_stack(guillotine_session_t *, 
		    sanson_arg_stack_t *, 
		    char *);
int pre_exec(guillotine_session_t *, sanson_paquet_t *, char *, char *);

/* terminal.c */
void do_sigchld();
void do_sigint() ;
int init_terminal(guillotine_session_t *);

/* lame.c */
void guillotine_lame_init(guillotine_session_t *);
int guillotine_lame_add(guillotine_session_t *, sanson_paquet_t *);
int guillotine_lame_del(guillotine_session_t *, char *);
sanson_paquet_t *guillotine_lame_name_get(guillotine_session_t *, char *);
int guillotine_lame_ls(guillotine_session_t *);
int guillotine_lame_stat(guillotine_session_t *);

#endif
