#ifndef __HEADMAN_H__
#define __HEADMAN_H__

//#define __DEBUG_HEADMAN__				1

#define SERVER_NAME "guillotine"

/* src/headman */

typedef struct s_headman
{
  int current_session; /* number of the current session */
  int next_new_session; /* number of the next new session */
  
  TAILQ_HEAD(s_tailq_session, s_headman_session) tailq_session_t; /* list of sessions */
  TAILQ_HEAD(s_tailq_command, pCommand) tailq_command_t; /* list of commands */
} headman_t;

typedef struct s_headman_session
{
  sanson_session_t sst;
  int id; /* session ID */
  int proto; /* PROTO_IPV4 ou PROTO_IPV6 */

  char *local_ipv4;
  char *local_ipv6;
  char *remote_ipv4;
  char *remote_ipv6;

  int local_port;
  int remote_port;

  TAILQ_ENTRY(s_headman_session)  next;
} headman_session_t;

#define SCOPE_ALL 0 /* used to know in which scope we are */
#define SCOPE_SESSION 1 /* used to know in which scope we are */

#define HEADMAN_PROMPT			"headman"

/* network.c */

#define PROTO_IPV4 0
#define PROTO_IPV6 1

int listen_tcp(headman_session_t *, int, int);
int connect_tcp(headman_session_t *);

/* shell.c */

int parse_headman_shell(headman_t *, char *);
void run_headman_shell(headman_t *);
void init_headman_shell(headman_t *);
char *print_prompt(headman_t *);

/* command.c */

#define COMMANDS_DIR "commands/"

typedef struct pCommand {
  char *name;
  void *handle;
  int scope; /* SCOPE_SESSION or SCOPE_ALL */
  
  void (*init)();
  void (*cleanup)();
  void (*run)();
  void (*help)();
  
  TAILQ_ENTRY(pCommand) next;
} pCommand;

pCommand *find_command(struct s_tailq_command *, char *);
int load_commands(struct s_tailq_command *);
void load_command(struct s_tailq_command *, char *);
int unload_commands(struct s_tailq_command *);
int unload_command(struct s_tailq_command *, char *);
int exec_command(headman_t *, char *, char *);

/* session.c */

headman_session_t *get_current_session(headman_t *);
void headman_session_init(headman_t *, headman_session_t *);
int get_paquet_list(sanson_session_t *);

#endif
