#ifndef __SANSON_H__
#define __SANSON_H__

/* Include */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <ctype.h>
#include <signal.h>
#include <errno.h>
#include <termios.h>

#include <elf.h>
#include <sys/mman.h>

#include <sys/queue.h>
#include "queue_default.h"

#include <sys/ptrace.h>
#include <sys/user.h>

#include <stdarg.h>

#include "xyssl/aes.h"
#include "xyssl/sha2.h"
#include "xyssl/net.h"
#include "xyssl/havege.h"
#include "xyssl/certs.h"
#include "xyssl/x509.h"
#include "xyssl/ssl.h"

#include "sanson_x.h"
#include "sanson_enum.h"
#include "sanson_struct.h"

#define SANSON_VERSION					"0.5"
#define SANSON_CODENAME					"bean"

/* Define DEBUG */

#define __DEBUG_SANSON__				1
#define __DEBUG_NETWORK__				1
#define __DEBUG_ALLOC__					0

/******** src/sanson ********/

/**** session.c ****/

/* Inititalise new session */
int sanson_session_init(sanson_session_t *);

int sanson_session_paquet_add(sanson_session_t *, sanson_paquet_t *);
int sanson_session_paquet_del(sanson_session_t *, char *);
/* Alloc a new binary (char *) to a session */
sanson_paquet_t *sanson_session_paquet_new(sanson_session_t *, char *);
/* Free a session */
int sanson_session_free(sanson_session_t *);
int sanson_session_free_without_paquet(sanson_session_t *, sanson_paquet_t *);

/* Display the header */
int sanson_paquet_header_view(sanson_paquet_header_t *);
/* Display the paquet */
int sanson_paquet_view(sanson_paquet_t *);
/* Display the paquet which have the name (char *) in the session */
int sanson_session_paquet_name_view(sanson_session_t *, char *);

/* Get a paquet by name in a session */
sanson_paquet_t * sanson_session_paquet_name_get(sanson_session_t *, char *);

/**** paquet.c ****/

int paquet_crypt(sanson_session_t *, char *, size_t, char *, size_t);
int paquet_uncrypt(sanson_session_t *, char *, size_t, char *, size_t);
int make_paquet_crypt(sanson_session_t *, sanson_paquet_t *, int);
/* Make a new paquet with a static binary */
int make_paquet_binary_static(sanson_session_t *, sanson_paquet_t *, int);
int make_paquet_binary_dyn(sanson_session_t *, sanson_paquet_t *, int);
/* Make a new paquet ? static : dynamic */
int sanson_make_paquet_binary(sanson_session_t *, char *, int, int);
int sanson_make_virtual_paquet_binary(sanson_session_t *, char *, int, int, int, unsigned long, void *);


/**** network.c ****/

/* Initialise network ? BIND_SOCKET, CONNECT_BACK ... */
int sanson_network_init(sanson_session_t *);
/* Close the network */
int sanson_network_close(sanson_session_t *);

int sanson_network_usessl(sanson_session_t *);
int sanson_network_notusessl(sanson_session_t *);
int sanson_network_init_server(sanson_session_t *, char *, char *, char *);
int sanson_network_init_client(sanson_session_t *, char *, char *, char *);

/* Set the fd in the session */
int sanson_network_fd_set(sanson_session_t *, int);
/* Return the fd in the session */
int sanson_network_get_fd(sanson_session_t *);
/* Return the real fd */
int sanson_network_get_realfd(sanson_session_t *);

/* Accept a connection */
int sanson_network_accept(sanson_session_t *);

/* Receive a paquet */
int sanson_network_recv(sanson_session_t *, void *, size_t, int);
/* Send a paquet */
int sanson_network_send(sanson_session_t *, void *, size_t, int);

/* Wrapper to cmd : send, list, del, exit */
int sanson_network_send_cmd_send(sanson_session_t *, char *);
int sanson_network_send_cmd_list(sanson_session_t *, char *);
int sanson_network_send_cmd_del(sanson_session_t *, char *);
int sanson_network_send_cmd_exit(sanson_session_t *, char *);
int sanson_network_send_cmd_exec(sanson_session_t *, char *);
int sanson_network_send_argvenv(sanson_session_t *, char *, char *);
int sanson_network_send_cmd_option(sanson_session_t *, char *);
int sanson_network_send_cmd_killguillotine(sanson_session_t *, char *);

int send_paquet_header(sanson_paquet_header_t *, int);
int send_paquet(sanson_session_t *, char *, int);
void * recv_paquet(sanson_session_t *, char *, int);

#if __DEBUG_NETWORK__
int sanson_network_debug(sanson_session_t *, unsigned char *, unsigned int, size_t);
#else
#define sanson_network_debug(a, b, c, d)           while (0) {}
#endif

/**** maps.c ****/

int view_binary_maps(void);

int add_virtual_target(virtual_target_t *, target_t *);
int view_virtual_target(virtual_target_t *);

int ul_maps_down_at(sanson_session_t *, sanson_execution_t *);
int ul_maps_down_procmaps(sanson_session_t *,  sanson_execution_t *);
int sanson_ul_maps_down(sanson_session_t *,  sanson_execution_t *);

int sanson_ul_make_vt(sanson_session_t *, sanson_paquet_t *,  sanson_execution_t *);
int ul_make_vt_classic(sanson_session_t *, sanson_paquet_t *, virtual_target_t *);

int ul_get_text_elf(sanson_session_t *, sanson_execution_t *, unsigned long *, unsigned long *);
int sanson_ul_get_text(sanson_session_t *, sanson_execution_t *, unsigned long *, unsigned long *);

int ul_get_libc_maps(sanson_session_t *, sanson_execution_t *, unsigned long *, unsigned long *);
int sanson_ul_get_libc(sanson_session_t *, sanson_execution_t *, unsigned long *, unsigned long *);

int ul_get_stack_maps(int, unsigned long *, unsigned long *);

/**** ul_exec.c ****/

int userland_execve_paquet_uncrypt(sanson_session_t *, sanson_paquet_t *);

int sanson_userland_execve(sanson_session_t *,
			   sanson_paquet_t *, 
			   sanson_arg_stack_t *,
			   sanson_arg_stack_t *);

/**** ul_exec_static.c ****/

int userland_execve_paquet_static(sanson_session_t *,
				  sanson_paquet_t *, 
				  sanson_arg_stack_t *,
				  sanson_arg_stack_t *,
				  sanson_execution_t *);


/**** ul_exec_dyn.c ****/

int userland_execve_view_dynsym(sanson_session_t *, sanson_paquet_t *, sanson_execution_t *);
int userland_execve_view_reloc(sanson_session_t *, sanson_execution_t *);
int userland_execve_view_plt(sanson_session_t *, sanson_execution_t *);
int userland_execve_free_reloc(sanson_session_t *, sanson_execution_t *);
int userland_execve_free_plt(sanson_session_t *, sanson_execution_t *);
int userland_execve_free_sym(sanson_session_t *, sanson_execution_t *);

Elf32_Sym *userland_execve_get_name(sanson_paquet_t *, 
				    sanson_execution_t *,
				    int, int, int);
int userland_execve_feed_elf(sanson_paquet_t *, sanson_execution_t *);
int userland_execve_get_libc_symbols(sanson_session_t *, sanson_execution_t *);

int userland_execve_make_reloc(sanson_session_t *, sanson_execution_t *);

unsigned long userland_execve_get_main(sanson_session_t *, 
				       sanson_paquet_t *, 
				       sanson_execution_t *, 
				       char *);

int userland_execve_get_reloc(sanson_session_t *, sanson_execution_t *, unsigned long);
int userland_execve_get_relplt(sanson_session_t *, sanson_paquet_t *, sanson_execution_t *);
int userland_execve_get_reldyn(sanson_session_t *, sanson_paquet_t *, sanson_execution_t *);
int userland_execve_get_processplt(sanson_session_t *, sanson_paquet_t *, sanson_execution_t *);
int userland_execve_get_processsymbs(sanson_session_t *, sanson_paquet_t *, sanson_execution_t *);
int userland_execve_get_processrel(sanson_session_t *, sanson_paquet_t *, sanson_execution_t *);
int userland_execve_get_processrelsymbs(sanson_session_t *, sanson_paquet_t *, sanson_execution_t *);
int userland_execve_patch_got(sanson_session_t *, sanson_paquet_t *, sanson_execution_t *);

int userland_execve_paquet_dyn(sanson_session_t *,
			       sanson_paquet_t *, 
			       sanson_arg_stack_t *,
			       sanson_arg_stack_t *,
			       sanson_execution_t *);

/**** stack.c ****/

unsigned long sanson_locate_stack_bruteforce(void);
unsigned long sanson_locate_stack(sanson_session_t *);
unsigned long find_name_stack(sanson_session_t *, unsigned long);
unsigned long sanson_find_name(sanson_session_t *, unsigned long);

int build_stack_static(sanson_session_t *, 
		       sanson_execution_t *,
		       sanson_arg_stack_t *,
		       sanson_arg_stack_t *);

/**** misc.c ****/

/* man strtok_r */ 
unsigned int sanson_strspn( char *,  char *);
char * sanson_strpbrk( char *,  char *);
char * sanson_strtok_r(char *,  char *, char **);

/* Dump a memory zone, hexa and ascii */
void sanson_hexdump(unsigned char *, unsigned int, size_t);
/* Setup the size to dump ? -1 => all  : X */
int sanson_hexdump_set(sanson_session_t *, int);

#if __DEBUG_SANSON__
void sanson_debug(const char *fmt,...);
#else
#define sanson_debug(fmt, ...)           while (0) {}
#endif

void *find_pattern(const void *, int, const void *, int);

/**** alloc.c ****/

/* Alloc/Free a new memory zone */
void * sanson_alloc(sanson_session_t *, int);
int sanson_free(sanson_session_t *, void *, int);
/* Setup the allocation */
int sanson_session_alloc_set(sanson_session_t *, int);

/**** inject.c ****/

int sanson_inject(sanson_session_t *, sanson_paquet_t *, sanson_inject_t *);

#endif
