#ifndef __SANSON_STRUCT_H__
#define __SANSON_STRUCT_H__

/* Define size */

#define SANSON_SIZE_STRING_NAME				64
#define SANSON_SIZE_STRING_ARG				256
#define SANSON_SIZE_STRING				256

/* Define protocol */

#define SANSON_SIZE_CMD					64
#define SANSON_SIZE_EXEC				128
#define SANSON_CMD_SEND					"SEND"
#define SANSON_CMD_DEL					"DEL"
#define SANSON_CMD_EXEC					"EXEC"
#define SANSON_CMD_LIST					"LIST"
#define SANSON_CMD_OPTION				"OPTION"
#define SANSON_CMD_EXIT					"EXIT"
#define SANSON_CMD_KILLGUILLOTINE			"KILLMEBABY"

#define ALIGN(k, v) (((k)+((v)-1))&(~((v)-1)))

#define SANSON_ALLOC_PROT_DEFAULT			PROT_READ|PROT_EXEC|PROT_WRITE
#define SANSON_ALLOC_MAP_DEFAULT			MAP_PRIVATE|MAP_ANONYMOUS
#define SANSON_PAGE_SIZE_DEFAULT			0x1000
#define SANSON_EAT_STACK_DEFAULT			65536

typedef struct s_addr
{

  unsigned long addr;
  int addrl;
  int action;
  int flags;
  
  TAILQ_ENTRY(s_addr)            entries;

} addr_t;

TAILQ_HEAD(s_tailq_addr, s_addr);

typedef struct s_target
{

  unsigned long addr;
  unsigned long addrl;
  int flags;
  int action;
  unsigned long orig;
  unsigned long origl;

} target_t;

typedef struct s_virtual_target
{

  char buffer[2048];
  int lenc;
  int nb;

} virtual_target_t;

/* new struct to manage arguments */
typedef struct s_sanson_argv
{

  char		      	name[SANSON_SIZE_STRING_ARG];	/* Argument name */
  
  TAILQ_ENTRY(s_sanson_argv)		entries; /* Next argv ! */
  
} sanson_argv_t;


/* Define new tailq struct to manage s_sanson_env and s_sanson_argv */
TAILQ_HEAD(s_tailq_sanson_argv, s_sanson_argv);

typedef struct s_sanson_arg_stack
{

  struct s_tailq_sanson_argv		argv;
  int					argc;
  int					size;
  
} sanson_arg_stack_t;

typedef struct s_sanson_func_sym
{

  Elf32_Sym		*sym;
  unsigned long		addr;
  char			*buffer;

  TAILQ_ENTRY(s_sanson_func_sym)		entries; /* Next func_sym ! */
  
} sanson_func_sym_t;

TAILQ_HEAD(s_tailq_sanson_func_sym, s_sanson_func_sym);

typedef struct s_sanson_func_plt
{

  Elf32_Sym		*sym;
  unsigned long		addr;
  char			*buffer;
  int			idx;

  TAILQ_ENTRY(s_sanson_func_plt)		entries; /* Next func_plt ! */
  
} sanson_func_plt_t;

TAILQ_HEAD(s_tailq_sanson_func_plt, s_sanson_func_plt);

typedef struct s_sanson_func_reloc
{

  Elf32_Rel		*rel;
  Elf32_Sym		*sym;
  char			*buffer;
  unsigned long		off;
  int			space;
  int			type;

  TAILQ_ENTRY(s_sanson_func_reloc)		entries; /* Next func_reloc ! */
  
} sanson_func_reloc_t;

TAILQ_HEAD(s_tailq_sanson_func_reloc, s_sanson_func_reloc);

typedef struct s_sanson_execution
{

  unsigned long				st_top;
  unsigned long				esp;
  int					argsize;

  int					idxstrtab;
  int					idxsymtab;
  int					idxrelplt;
  int					idxreldyn;
  int					idxplt;
  int					idxdynsym;
  int					idxdynstr;
  int					idxgotplt;

  struct s_tailq_sanson_func_reloc	stsfr;
  struct s_tailq_sanson_func_plt	stsfp;
  struct s_tailq_sanson_func_sym	stsfs;
  int					nbsym;
  int					nbrelplt;
  int					nbreldyn;

  unsigned long				process_text_begin;
  unsigned long				process_text_end;
  unsigned long				process_libc_begin;
  unsigned long				process_libc_end;

  void					*mmaptr;
  unsigned long				entry;

  virtual_target_t			vtt;

} sanson_execution_t;

typedef struct s_sanson_inject
{

  int pid;
  struct user_regs_struct regs;

} sanson_inject_t;

/* new struct to manage library */
typedef struct s_sanson_paquet_lib
{

  unsigned long			size;	/* strlen(name.so) ! */
  void *			lib;	/* Address of the lib */

  TAILQ_ENTRY(s_sanson_paquet_lib)		entries; /* next lib ! */

} sanson_paquet_lib_t;


/* new struct to manage paquet's header */
typedef struct s_sanson_paquet_header
{

  unsigned long			total; /* Total size of the header */

  char				name[SANSON_SIZE_STRING_NAME]; /* Paquet's name */

  unsigned short		type; /* Type of the paquet ? static : dynamic */
  unsigned short		compress; /* Is it compressed ? */
  unsigned short		crypt; /* Is it crypted ? */

  unsigned long			size; /* Size of the binary */
  
  int				nb; /* strlen(tailq_lib) */

} sanson_paquet_header_t;


/* new struct to manage paquet */
typedef struct s_sanson_paquet
{
  sanson_paquet_header_t	spht; /* Header !! */

  void *			binary; /* Address of the binary */
  sanson_paquet_lib_t *		libs; /* Library ? yes : no */

  TAILQ_ENTRY(s_sanson_paquet)		entries; /* next paquet ! */

} sanson_paquet_t;


/* new struct to manage session */
typedef struct s_sanson_session
{

  int			fd;
  int			forced_fd;

  int			network; /* Connection type (bind socket, connect back or find socket) */
  int			hexdump; /* -1 => dump all, X => dump X */

  int			alloc; /* Type of allocation  (mmap, malloc, ...)*/
  int			ul_maps_down; /* Method used to print the mapping */
  int			ul_make_vt; /* Method to construct virutal targets */
  int			ul_exec_dyn; /* Method to execute a ET_DYN binary */
  int			find_name; /* Method to find program's name */
  int			random; /* Randomization */
  int			get_libc; /* Method to get the libc */
  int			get_text; /* Method to get the .text */
  int			alloc_inj; /* Method to inject a ET_DYN */
  int			alloc_getinj; /* Method to get the address of allocation */

  unsigned long		page_size; /* Length of the page size */

  int			alloc_flags; /* Allocation flags */
  int			alloc_map; /* Allocation map */

  unsigned long		eat_stack; /* Length to eat in the stack */

  int			swap; /* Not swap */
  unsigned long		len_swap; /* Lenght of the lock area */

  int			ssl; /* Use ssl */
  
  havege_state		hs;
  ssl_context		ssl_ctx;
  ssl_session		ssn;
  x509_cert		srvcert;
  rsa_context		rsa;

  x509_cert		cacert;
  x509_cert		clicert;

  unsigned char		key[512];
  int			keylen;

  TAILQ_HEAD(s_tailq_spt, s_sanson_paquet) tailq_spt_t;

} sanson_session_t;

#endif
