#ifndef __SANSON_X_H__
#define __SANSON_X_H__

/**
 * Safe malloc() 
 */
#define         XMALLOC(a, b, c)	                    \
do                                                          \
{                                                           \
  if ((a = malloc(b)) == NULL)                              \
	{						    \
	  perror("malloc");				    \
	  return c;					    \
	}						    \
}                                                           \
while (0)

/**
 * Safe free() ??? 
 */
#define         XFREE(a, b)		                    \
do                                                          \
{							    \
  free(a);						    \
}                                                           \
while (0)

/**
 * Safe open() 
 */
#define         XOPEN(a, b, c, d, e)                        \
do                                                          \
{                                                           \
  if ((a = open(b, c, d)) < 0)                              \
	{						    \
	  perror("open");				    \
	  return e;					    \
	}						    \
}                                                           \
while (0)

/**
 * Safe recv() 
 */
#define         XRECV(a, b, c, d, e)                        \
do                                                          \
{                                                           \
  if (recv(a, b, c, d) != c)				    \
	{						    \
	  perror("recv");				    \
	  return e;					    \
	}						    \
}                                                           \
while (0)

/**
 * Safe send() 
 */
#define         XSEND(a, b, c, d, e)                        \
do                                                          \
{                                                           \
  if (send(a, b, c, d) != c)				    \
	{						    \
	  perror("send");				    \
	  return e;					    \
	}						    \
}                                                           \
while (0)

/**
 * Safe read() 
 */
#define         XREAD(a, b, c, d)                           \
do                                                          \
{                                                           \
  if (read(a, b, c) != c)				    \
	{						    \
	  perror("read");				    \
	  return d;					    \
	}						    \
}                                                           \
while (0)

/**
 * Safe lseek() 
 */
#define         XSEEK(a, b, c, d)                           \
do                                                          \
{                                                           \
  if (lseek(a, b, c) == (off_t) -1)			    \
	{						    \
	  perror("lseek");				    \
	  return d;					    \
        }						    \
}                                                           \
while (0)

/**
 * Safe write 
 */
#define         XWRITE(a, b, c, d)                          \
do                                                          \
{                                                           \
  if (write(a, b, c) != c)				    \
	{						    \
	  perror("write");				    \
	  return d;					    \
	}						    \
							    \
 if (fsync(a))						    \
	{						    \
	  perror("fsync");				    \
	  return d;					    \
	}						    \
}                                                           \
while (0)

/**
 * Safe close 
 */
#define         XCLOSE(a, b)                                \
do                                                          \
{                                                           \
  if (close(a))						    \
	{						    \
	  perror("close");				    \
	  return b;					    \
	}						    \
}	                                                    \
while (0)

/**
 * @brief Safe mmap
 */
#define XMMAP(a, b, c, d, e, f, g, h)                       \
do                                                          \
{                                                           \
  if ((a = mmap(b, c, d, e, f, g)) == MAP_FAILED)	    \
	{						    \
	  perror("mmap");				    \
	  return h;					    \
	}						    \
}                                                           \
while (0)

/**
 * @brief Safe munmap
 */
#define XMUNMAP(a, b, c)                                    \
do                                                          \
{                                                           \
  if (munmap(a, b) == -1)				    \
	{						    \
	  perror("munmap");				    \
	  return c;					    \
	}						    \
}                                                           \
while(0)

/**
 * @brief Safe msync
 */
#define XMSYNC(a, b, c, d)                                  \
do                                                          \
{                                                           \
  if (msync(a, b, c) == -1)				    \
	{						    \
	  perror("msync");				    \
	  return d;					    \
	}						    \
}                                                           \
while(0)


/**
 * Safe fstat() 
 */
#define         XFSTAT(a, b, c)                             \
do                                                          \
{                                                           \
  if (fstat(a, b) == -1)				    \
	{						    \
	  perror("fstat");				    \
	  return c;					    \
	}						    \
}                                                           \
while (0)

#define         XMLOCK(a, b, c)                             \
do                                                          \
{                                                           \
  if (mlock(a, b) == -1)				    \
	{						    \
	  perror("mlock");				    \
	  return c;					    \
	}						    \
}                                                           \
while (0)

#define         XMUNLOCK(a, b, c)                           \
do                                                          \
{                                                           \
  if (munlock(a, b) == -1)				    \
	{						    \
	  perror("mlock");				    \
	  return c;					    \
	}						    \
}                                                           \
while (0)

#define         XCMP(a, b, c)                               \
do                                                          \
{                                                           \
  if (a == b)						    \
	{						    \
	  return c;					    \
	}						    \
}                                                           \
while (0)

#endif
