#define _GNU_SOURCE 500

#include "sanson.h"
#include "guillotine.h"

int pre_build_stack(guillotine_session_t *gst, 
		    sanson_arg_stack_t *sast, 
		    char *value)
{
  int size;
  char *p;
  sanson_argv_t *tmpargv;

  XCMP(sast, NULL, -1);
  
  sast->argc = size = 0;

  if (value)
    {
      p = value;
      while (*p != '\0')
	{
	  if (*p == ' ')
	    {
	      while (*p == ' ') p++;
	      if (*p != '\0')
		sast->argc++;
	    }
	  else
	    p++;
	}
      sast->argc++; /* NULL */

      p = value;
      while (*p != '\0')
	{
	  if (*p == ' ')
	    {
	      tmpargv = sanson_alloc(&gst->sst, sizeof(sanson_argv_t));
	      memset(tmpargv, '\0', sizeof(sanson_argv_t));
	      memcpy(tmpargv->name, p-size, size);
	      sast->size += size;
	      TAILQ_INSERT_TAIL(&sast->argv, tmpargv, entries);

	      size = 0;
	      while (*p == ' ') p++;
	    }
	  else
	    {
	      p++;
	      size++;
	    }
	}
   
      if (size != 0)
	{
	  tmpargv = sanson_alloc(&gst->sst, sizeof(sanson_argv_t));
	  memset(tmpargv, '\0', sizeof(sanson_argv_t));
	  memcpy(tmpargv->name, p-size, size);
	  sast->size += size;
	  TAILQ_INSERT_TAIL(&sast->argv, tmpargv, entries);
	}
    }

  return 0;
}


void copie_entrees_sorties(int fd, int sock)
{
  int max;
  fd_set set;
  char buffer[4096];
  int nb_lus;

  max = sock < fd ? fd : sock;

  while(1)
    {
      FD_ZERO(&set);
      FD_SET(sock, &set);
      FD_SET(fd, &set);

      if (select(max+1, &set, NULL, NULL, NULL) < 0)
	break;

      if (FD_ISSET(sock, &set))
	{
	  if ((nb_lus = read(sock, buffer, 4096)) >= 0)
	    write(fd, buffer, nb_lus);
	  else
	    break;
	}

      if (FD_ISSET(fd, &set))
	{
	  if ((nb_lus = read(fd, buffer, 4096)) >= 0)
	    write(sock, buffer, nb_lus);
	  else
	    break;
	}
    }
}


int pre_exec(guillotine_session_t *gst, sanson_paquet_t *spt, char *argv, char *env)
{
  sanson_arg_stack_t sargv, senv;
  int pnum;

  int fd_maitre;
  int fd_esclave;

  /*******/
  struct termios termios_stdin;
  struct termios termios_maitre;
  char *nom_esclave;
  /******/

  sanson_argv_t *tmpargv;

  XCMP(gst, NULL, -1);
  XCMP(spt, NULL, -1);

  memset(&sargv, '\0', sizeof(sanson_arg_stack_t));
  memset(&senv, '\0', sizeof(sanson_arg_stack_t));

  /* Initialize TAILQ argv and envp */

  TAILQ_INIT(&sargv.argv);
  TAILQ_INIT(&senv.argv);

  /* Hey baby, pop my stack and my env !!!! */
  pre_build_stack(gst, &sargv, argv);
  pre_build_stack(gst, &senv, env);

  if (gst->sst.find_name != SANSON_FIND_NAME_ARG0)
    {
      if (gst->name != NULL)
	{
	  tmpargv = sanson_alloc(&gst->sst, sizeof(sanson_argv_t));
	  memset(tmpargv, '\0', sizeof(sanson_argv_t));
	  memcpy(tmpargv->name, gst->name, strlen(gst->name));
	  sargv.size += strlen(gst->name);
	  sargv.argc++;
	  TAILQ_INSERT_HEAD(&sargv.argv, tmpargv, entries);
	}
      else
	{
	  tmpargv = sanson_alloc(&gst->sst, sizeof(sanson_argv_t));
	  memset(tmpargv, '\0', sizeof(sanson_argv_t));
	  memcpy(tmpargv->name, spt->spht.name, strlen(spt->spht.name));
	  sargv.size += strlen(spt->spht.name);
	  sargv.argc++;
	  TAILQ_INSERT_HEAD(&sargv.argv, tmpargv, entries);
	}
    }

  /*******/
  if ((fd_maitre = getpt()) < 0)
    {
      perror("pas de pseudo tty unix 98 dispos\n");
      exit (1);
    }

  grantpt(fd_maitre);
  unlockpt(fd_maitre);
  nom_esclave = ptsname(fd_maitre);
  tcgetattr (STDIN_FILENO, &termios_stdin);
  /*******/

  /* Fork me !!! Fork me !!! */
  
  pnum = fork();
  switch (pnum)
    {
    case 0 : 
      /*********/
      close(fd_maitre);
      setsid();
      if ((fd_esclave = open(nom_esclave, O_RDWR)) < 0)
	{
	  perror("open");
	  exit(1);
	}

      tcsetattr(fd_esclave, TCSANOW, &termios_stdin);
      dup2(fd_esclave, STDIN_FILENO);
      dup2(fd_esclave, STDOUT_FILENO);
      dup2(fd_esclave, STDERR_FILENO);
      /*********/

      /* Userland execve paquet !!! see sanson/ul_exec.c */
      sanson_userland_execve(&gst->sst, spt, &sargv, &senv);
      break;
    case -1 : 
      perror("fork");
      exit (1);
    default :
      /**********/
      tcgetattr(fd_maitre, &termios_maitre);
      cfmakeraw(&termios_maitre);
      tcsetattr(fd_maitre, TCSANOW, &termios_maitre);
      copie_entrees_sorties(fd_maitre, gst->sst.forced_fd);
      exit(0);
    }

  return 0;
}
