#include "sanson.h"
#include "guillotine.h"

void guillotine_lame_init(guillotine_session_t *gst)
{
  sanson_session_init(&gst->sst);

  sanson_session_alloc_set(&gst->sst, SANSON_ALLOC_MMAP);

  gst->stats.memory = 0;
  gst->stats.nb = 0;
  gst->name = NULL;
}

int guillotine_lame_add(guillotine_session_t *gst, sanson_paquet_t *lame)
{
  sanson_session_paquet_add(&gst->sst, lame);

  gst->stats.memory += lame->spht.total;
  gst->stats.nb += 1;
 
  return 0;
}

int guillotine_lame_del(guillotine_session_t *gst, char *name)
{
  sanson_paquet_t *tmp;

  XCMP(name, NULL, -1);

  for (tmp = gst->sst.tailq_spt_t.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next)
    {
      if (!strcmp(tmp->spht.name, name))
	{
	  gst->stats.memory -= tmp->spht.total;
	  gst->stats.nb -= 1;
	  
	  return sanson_session_paquet_del(&gst->sst, name);
	}
    }

  return -1;
}

sanson_paquet_t *guillotine_lame_name_get(guillotine_session_t *gst, char *name)
{
  sanson_paquet_t *tmp;

  tmp = sanson_session_paquet_name_get(&gst->sst, name);

  return tmp;
}

int guillotine_lame_ls(guillotine_session_t *gst)
{
  sanson_paquet_t *tmp;
  
  for (tmp = gst->sst.tailq_spt_t.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next)
    {
      printf("PAQUET @ 0x%lx\n", (unsigned long) tmp);
      sanson_paquet_header_view(&tmp->spht);
      printf("\t BINARY 0x%lx\n", (unsigned long) tmp->binary);
      printf("\t LIBS 0x%lx\n", (unsigned long) tmp->libs);
    }
  
  return 0;
}

int guillotine_lame_stat(guillotine_session_t *gst)
{
  fprintf(stdout, "[+] STAT :\n"); 
  fprintf(stdout, "\t [+] MEMORY %d\n", (unsigned int)gst->stats.memory);
  fprintf(stdout, "\t [+] COUNTER %d\n", gst->stats.nb);

  return 0;
}
