#include "sanson.h"

#include <sys/ioctl.h>

#ifdef ENABLE_SSL
#define DEBUG_LEVEL 0

#define SERVER_NAME "guillotine"

extern char ca_crt[];
extern char headman_crt[];
extern char headman_key[];

void my_debug( void *ctx, int level, char *str )
{
    if( level < DEBUG_LEVEL )
    {
        fprintf( (FILE *) ctx, "%s", str );
        fflush(  (FILE *) ctx  );
    }
}
#endif

int main(int argc, char *argv[])
{
  sanson_session_t test;
  int ret;

  char buffer[4096];
  int nb_lus;
  struct termios termios_stdin;
  struct termios termios_raw;
  fd_set set;

  sanson_session_init(&test);
  sanson_network_init(&test);

#ifdef ENABLE_SSL
  sanson_network_usessl(&test);
  sanson_network_init_client(&test, ca_crt, headman_crt, headman_key);
#endif

  sanson_session_alloc_set(&test, SANSON_ALLOC_MALLOC);
  
  if (( ret = net_connect(&test.fd, "127.0.0.1", 2002)) != 0)
    {
      printf( " failed\n  ! net_connect returned %d\n\n", ret );
      return -1;
    }

#ifdef ENABLE_SSL
  havege_init( &test.hs );

  if( ( ret = ssl_init( &test.ssl_ctx ) ) != 0 )
    {
      printf( " failed\n  ! ssl_init returned %d\n\n", ret );
      return -1;
    }
  
  ssl_set_endpoint( &test.ssl_ctx, SSL_IS_CLIENT );
  ssl_set_authmode( &test.ssl_ctx, SSL_VERIFY_OPTIONAL );
  
  ssl_set_rng( &test.ssl_ctx, havege_rand, &test.hs );
  ssl_set_dbg( &test.ssl_ctx, my_debug, stdout );
  ssl_set_bio( &test.ssl_ctx, net_recv, &test.fd,
	       net_send, &test.fd );
  
  ssl_set_ciphers( &test.ssl_ctx, ssl_default_ciphers );
  ssl_set_session( &test.ssl_ctx, 1, 600, &test.ssn );
  
  ssl_set_ca_chain( &test.ssl_ctx, &test.cacert, SERVER_NAME );
  ssl_set_own_cert( &test.ssl_ctx, &test.clicert, &test.rsa );

  ssl_set_hostname( &test.ssl_ctx, SERVER_NAME );

  while( ( ret = ssl_handshake( &test.ssl_ctx ) ) != 0 )
    {
      if( ret != XYSSL_ERR_NET_TRY_AGAIN )
        {
	  printf( " failed\n  ! ssl_handshake returned %d\n\n", ret );
	  return -1;
        }
    }
  
  printf( " ok\n    [ Cipher is %s ]\n",
	  ssl_get_cipher( &test.ssl_ctx ) );
  
  printf( "  . Verifying peer X.509 certificate..." );
  
  if( ( ret = ssl_get_verify_result( &test.ssl_ctx ) ) != 0 )
    {
      printf( " failed\n" );
      
      if( ( ret & BADCERT_EXPIRED ) != 0 )
	printf( "  ! server certificate has expired\n" );
      
      if( ( ret & BADCERT_REVOKED ) != 0 )
	printf( "  ! server certificate has been revoked\n" );
      
      if( ( ret & BADCERT_CN_MISMATCH ) != 0 )
	printf( "  ! CN mismatch (expected CN=%s)\n", SERVER_NAME );
      
      if( ( ret & BADCERT_NOT_TRUSTED ) != 0 )
	printf( "  ! self-signed or not signed by a trusted CA\n" );
      
      printf( "\n" );
      return -1;
    }
  else
    printf( " ok\n" );
  
  printf( "  . Peer certificate information    ...\n" );
  printf( x509parse_cert_info( "      ", test.ssl_ctx.peer_cert ) );
#endif
  
  if (sanson_make_paquet_binary(&test, argv[1], SANSON_COMPRESS_NONE, SANSON_CRYPT_NONE) == 0)
    {
      sanson_session_paquet_name_view(&test, argv[1]);

      sanson_paquet_t * tmp = sanson_session_paquet_name_get(&test, argv[1]);
      
      sanson_network_send_cmd_exec(&test, argv[1]);
      if (argc == 2)
	sanson_network_send_argvenv(&test, NULL, NULL);
      else if (argc == 3)
	sanson_network_send_argvenv(&test, argv[2], NULL);
      else if (argc == 4)
	sanson_network_send_argvenv(&test, argv[2], argv[4]);


      struct termio temp_mode;

	if ( ! isatty(1) )
	  return -1;

	temp_mode.c_lflag=(ISIG|ICANON|ECHO|ECHOE);
	temp_mode.c_iflag=(BRKINT|IGNPAR| /* ISTRIP| */ ICRNL|IXON);
	temp_mode.c_oflag=(OPOST|ONLCR);
	temp_mode.c_cflag=(CS7|PARENB|CREAD);
	temp_mode.c_cc[VERASE]=('H'^64);
	temp_mode.c_cc[VKILL]=('U'^64);
	temp_mode.c_cc[VQUIT]=('\\'^64);
	temp_mode.c_cc[VINTR]=('C'^64);
	temp_mode.c_cc[VEOF]=('D'^64);
	
	if (ioctl(1, TCSETA, (char *) &temp_mode) < 0)
	  return -1;

      tcgetattr(STDIN_FILENO, &termios_stdin);
      tcgetattr(STDIN_FILENO, &termios_raw);
      cfmakeraw(&termios_raw);
      tcsetattr(STDIN_FILENO, TCSANOW, &termios_raw);

      while(1)
	{
	  FD_ZERO(&set);
	  FD_SET(test.fd, &set);
	  FD_SET(STDIN_FILENO, &set);
	  
	  if (select(test.fd+1, &set, NULL, NULL, NULL) < 0)
	    break;

	  if (FD_ISSET(test.fd, &set))
	    {
	      if ((nb_lus = read(test.fd, buffer, 4096)) <= 0)
		break;

	      write(STDOUT_FILENO, buffer, nb_lus);
	    }

	  if (FD_ISSET(STDIN_FILENO, &set))
	    {
	      if ((nb_lus = read(STDIN_FILENO, buffer, 4096)) <= 0)
		break;
	      write(test.fd, buffer, nb_lus);
	    }
	  if (ioctl(1, TCSETA, (char *) &temp_mode) < 0)
	    return -1;
	}

      tcsetattr(STDIN_FILENO, TCSANOW, &termios_stdin);
    }

  sanson_network_close(&test);

#ifdef ENABLE_SSL
  x509_free( &test.clicert );
  x509_free( &test.cacert );
  rsa_free( &test.rsa );
  ssl_free( &test.ssl_ctx );
  
  memset( &test.ssl_ctx, 0, sizeof( ssl_context ) );
#endif

  return 0;
}
