#include "sanson.h"

#ifdef ENABLE_SSL

#define DEBUG_LEVEL 0

#define SERVER_NAME "guillotine"

extern char ca_crt[];
extern char headman_crt[];
extern char headman_key[];

void my_debug( void *ctx, int level, char *str )
{
    if( level < DEBUG_LEVEL )
    {
        fprintf( (FILE *) ctx, "%s", str );
        fflush(  (FILE *) ctx  );
    }
}
#endif

int main(int argc, char *argv[])
{
  sanson_session_t test;
  int ret;
  char option[512];

  sanson_session_init(&test);
  sanson_network_init(&test);

#ifdef ENABLE_SSL
  sanson_network_usessl(&test);
  sanson_network_init_client(&test, ca_crt, headman_crt, headman_key);
#endif

  sanson_session_alloc_set(&test, SANSON_ALLOC_MALLOC);
 
    if (( ret = net_connect(&test.fd, "127.0.0.1", 2002)) != 0)
    {
      printf( " failed\n  ! net_connect returned %d\n\n", ret );
      return -1;
    }

#ifdef ENABLE_SSL
  havege_init( &test.hs );
  
  if( ( ret = ssl_init( &test.ssl_ctx ) ) != 0 )
    {
      printf( " failed\n  ! ssl_init returned %d\n\n", ret );
      return -1;
    }
  
  ssl_set_endpoint( &test.ssl_ctx, SSL_IS_CLIENT );
  ssl_set_authmode( &test.ssl_ctx, SSL_VERIFY_OPTIONAL );
  
  ssl_set_rng( &test.ssl_ctx, havege_rand, &test.hs );
  ssl_set_dbg( &test.ssl_ctx, my_debug, stdout );
  ssl_set_bio( &test.ssl_ctx, net_recv, &test.fd,
	       net_send, &test.fd );
  
  ssl_set_ciphers( &test.ssl_ctx, ssl_default_ciphers );
  ssl_set_session( &test.ssl_ctx, 1, 600, &test.ssn );
  
  ssl_set_ca_chain( &test.ssl_ctx, &test.cacert, SERVER_NAME );
  ssl_set_own_cert( &test.ssl_ctx, &test.clicert, &test.rsa );

  ssl_set_hostname( &test.ssl_ctx, SERVER_NAME );

  while( ( ret = ssl_handshake( &test.ssl_ctx ) ) != 0 )
    {
      if( ret != XYSSL_ERR_NET_TRY_AGAIN )
        {
	  printf( " failed\n  ! ssl_handshake returned %d\n\n", ret );
	  return -1;
        }
    }
  
  printf( " ok\n    [ Cipher is %s ]\n",
	  ssl_get_cipher( &test.ssl_ctx ) );
  
  printf( "  . Verifying peer X.509 certificate..." );
  
  if( ( ret = ssl_get_verify_result( &test.ssl_ctx ) ) != 0 )
    {
      printf( " failed\n" );
      
      if( ( ret & BADCERT_EXPIRED ) != 0 )
	printf( "  ! server certificate has expired\n" );
      
      if( ( ret & BADCERT_REVOKED ) != 0 )
	printf( "  ! server certificate has been revoked\n" );
      
      if( ( ret & BADCERT_CN_MISMATCH ) != 0 )
	printf( "  ! CN mismatch (expected CN=%s)\n", SERVER_NAME );
      
      if( ( ret & BADCERT_NOT_TRUSTED ) != 0 )
	printf( "  ! self-signed or not signed by a trusted CA\n" );
      
      printf( "\n" );
      return -1;
    }
  else
    printf( " ok\n" );
  
  printf( "  . Peer certificate information    ...\n" );
  printf( x509parse_cert_info( "      ", test.ssl_ctx.peer_cert ) );
#endif

  memset(option, '\0', sizeof(option));
  snprintf(option,
	   sizeof(option),
	   "%s %s",
	   argv[1],
	   argv[2]);

  sanson_network_send_cmd_option(&test, option);

  sanson_network_send_cmd_exit(&test, NULL);

  sanson_network_close(&test);

#ifdef ENABLE_SSL
  x509_free( &test.clicert );
  x509_free( &test.cacert );
  rsa_free( &test.rsa );
  ssl_free( &test.ssl_ctx );
  
  memset( &test.ssl_ctx, 0, sizeof( ssl_context ) );
#endif

  return 0;
}
