#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>
#include <dirent.h>
#include <sys/queue.h>
#include "sanson.h"
#include "headman.h"

pCommand *find_command(struct s_tailq_command *clist_head, char *name)
{
  pCommand *current;

  TAILQ_FOREACH(current, clist_head, next) {
    if(!strcmp(current->name, name))
      return current;
  }

  return NULL;
}

int load_commands(struct s_tailq_command *clist_head)
{
  int nb = 0;
  DIR *dir;
  struct dirent * dir_entry;
  char *str_so = ".so";

  dir = opendir(COMMANDS_DIR);
  
  while( NULL != (dir_entry = readdir(dir)))
    {
      if(strstr(dir_entry->d_name, str_so))
	{
	  nb++;
	  load_command(clist_head, dir_entry->d_name);
	}
    }
  
  closedir(dir);

  return nb;
}

void load_command(struct s_tailq_command *clist_head, char *name)
{
  char *error;
  void *handle;
  pCommand *cmd;
  char *tname;

  cmd = find_command(clist_head, name);
  if(cmd != NULL)
    return;

  tname = (char *)malloc(strlen(name)+ strlen(COMMANDS_DIR) + 4);
  bzero(tname, strlen(name)+ strlen(COMMANDS_DIR) + 4);
  sprintf(tname, "%s%s", COMMANDS_DIR, name);

  handle = dlopen(tname, RTLD_NOW|RTLD_GLOBAL);
  if(handle == NULL)
    {
      printf("error1 %s\n", dlerror());
      return;
    }
  
  cmd = dlsym(handle, "mod");
  if((error = dlerror()) != NULL)
    {
      printf("error2 %s\n", error);
      dlclose(handle);
      return;
    }
  
  cmd->handle = handle;
  
  TAILQ_INSERT_HEAD(clist_head, cmd, next);

  cmd->init();
  
  free(tname);
}

int unload_commands(struct s_tailq_command *clist_head)
{
  pCommand *current;
  
  TAILQ_FOREACH(current, clist_head, next) {
    unload_command(clist_head, current->name);
  }
    
  return 0;
}

int unload_command(struct s_tailq_command *clist_head, char *name)
{
  pCommand *cmd;

  cmd = find_command(clist_head, name);
  if(cmd == NULL)
    return -1;
  
  cmd->cleanup();

  TAILQ_REMOVE(clist_head, cmd, next);  
  
  /*
   * FIXME : segfault
   */
  dlclose(cmd->handle);
    
  return 0;
}

int exec_command(headman_t *hdm, char *name, char *argv)
{
  pCommand *cmd;
  
  cmd = find_command(&hdm->tailq_command_t, name);
  if(cmd == NULL) 
    {
      fprintf(stdout, "-- command not found --\n");
      return 0;
    }
  
  cmd->run(hdm, argv);
  
  return 0;
}

