#include "sanson.h"
#include "headman.h"

void connect_init(void);
void connect_cleanup(void);
void connect_help(void);
void connect_run(headman_t *, char *);

pCommand mod =
{ "connect", NULL, SCOPE_SESSION, connect_init, connect_cleanup, connect_run, connect_help};

void connect_init(void)
{
#if __DEBUG_HEADMAN__
  fprintf(stdout, "-- CONNECT INIT --\n");
#endif
}

void connect_cleanup(void)
{
#if __DEBUG_HEADMAN__
  fprintf(stdout, "-- CONNECT CLEANUP --\n");
#endif
}

void connect_help(void)
{
  fprintf(stdout, "Establish a connection with the remote target.");
}

void connect_run(headman_t *hdm, char *argv)
{
  int port = 0;
  int proto = 4;
  int ssl = 1;
  char addr[SANSON_SIZE_STRING];
  char mode[4];
  char cmd[SANSON_SIZE_STRING];
  headman_session_t *session;

  if(hdm->current_session == -1)
    {
      fprintf(stdout, "-- your are not in a session --\n");
    }
  else 
    {
      memset(mode, '\0', 4);
      memset(cmd, '\0', SANSON_SIZE_STRING);
      fprintf(stdout, "connection mode ?\n\t* bind socket\t-> 'bind' (default)\n\t* find socket\t-> 'find'\n\t* connect back\t-> 'back'\nchoice: ");
      fflush(NULL);
      fgets(cmd, sizeof(cmd), stdin);
      sscanf(cmd, "%s", mode);

      session = get_current_session(hdm);

      if( strncmp(mode, "back", 4) == 0 )
	{
	  session->sst.network = SANSON_SOCKET_CONNECTBACK;

	  memset(cmd, '\0', SANSON_SIZE_STRING);
	  fprintf(stdout, "local port [2002]: ");
	  fflush(NULL);
	  fgets(cmd, sizeof(cmd), stdin);
	  sscanf(cmd, "%d", &port);
	  if(port == 0)
	    port = 2002;

	  memset(cmd, '\0', SANSON_SIZE_STRING);
	  fprintf(stdout, "protocol ?\n\t* IPv4 -> '4' (default)\n\t* IPv6 -> '6'\nchoice: ");
	  fflush(NULL);
	  fgets(cmd, sizeof(cmd), stdin);
	  sscanf(cmd, "%d", &proto);
	  if(proto == 0 || proto == 4)
	    proto = PROTO_IPV4;
	  else
	    proto = PROTO_IPV6;

	  fprintf(stdout, "-- local port : %d - mode : SOCKET_CONNECTBACK - proto : IPv%d --\n", port, (proto == PROTO_IPV4) ? 4 : 6);

	  if( listen_tcp(session, proto, port) < 0 )
	    {
	      perror("listen_tcp()");
	      return;
	    }
	  /* use select() for a max time */
	  fprintf(stdout, "-- listening for a connection --\n");
	  if( (session->sst.forced_fd = accept(session->sst.fd, NULL, NULL)) < 0 )
	    {
	      perror("accept()");
	      return;
	    }
	  fprintf(stdout, "-- connected to REMOTE_ADDR:REMOTE_PORT --\n");
	}
      else if( strncmp(mode, "find", 4) == 0 )
	{
	  session->sst.network = SANSON_SOCKET_FIND;

	  memset(cmd, '\0', SANSON_SIZE_STRING);
	  fprintf(stdout, "remote address [127.0.0.1]: ");
	  fflush(NULL);
	  fgets(cmd, sizeof(cmd), stdin);
	  sscanf(cmd, "%s", addr);
	  if(*cmd == '\n' || *cmd == ' ')
	    sprintf(addr, "127.0.0.1");
	  else
	    sscanf(cmd, "%s", addr);

	  memset(cmd, '\0', SANSON_SIZE_STRING);
	  fprintf(stdout, "remote port [2002]: ");
	  fflush(NULL);
	  fgets(cmd, sizeof(cmd), stdin);
	  sscanf(cmd, "%d", &port);
	  if(port == 0)
	    port = 2002;

	  memset(cmd, '\0', SANSON_SIZE_STRING);
	  fprintf(stdout, "protocol ?\n\t* IPv4 -> '4' (default)\n\t* IPv6 -> '6'\nchoice: ");
	  fflush(NULL);
	  fgets(cmd, sizeof(cmd), stdin);
	  sscanf(cmd, "%d", &proto);
	  if(proto == 0 || proto == 4)
	    proto = PROTO_IPV4;
	  else
	    proto = PROTO_IPV6;

	  fprintf(stdout, "-- remote address : %s:%d - mode : SOCKET_FIND - proto : IPv%d --\n", addr, port, (proto == PROTO_IPV4) ? 4 : 6);
	  fprintf(stdout, "-- connecting to %s:%d --\n", addr, port);

	  session->proto = proto;
	  session->remote_ipv4 = sanson_alloc(&session->sst, strlen(addr) + 1);
	  memset(session->remote_ipv4, '\0', strlen(addr) + 1);
	  memcpy(session->remote_ipv4, addr, strlen(addr));

	  session->remote_port = port;

	  if( connect_tcp(session) < 0 )
	    {
	      perror("connect_tcp()");
	      return;
	    }
	  fprintf(stdout, "-- connected to %s:%d --\n", addr, port);
	}
      else
	{
	  session->sst.network = SANSON_SOCKET_BIND;

	  memset(cmd, '\0', SANSON_SIZE_STRING);
	  fprintf(stdout, "remote address [127.0.0.1]: ");
	  fflush(NULL);
	  fgets(cmd, sizeof(cmd), stdin);
	  if(*cmd == '\n' || *cmd == ' ')
	    sprintf(addr, "127.0.0.1");
	  else
	    sscanf(cmd, "%s", addr);

	  session->remote_ipv4 = sanson_alloc(&session->sst, strlen(addr) + 1);
	  memset(session->remote_ipv4, '\0', strlen(addr) + 1);
	  memcpy(session->remote_ipv4, addr, strlen(addr));

	  memset(cmd, '\0', SANSON_SIZE_STRING);
	  fprintf(stdout, "remote port [2002]: ");
	  fflush(NULL);
	  fgets(cmd, sizeof(cmd), stdin);
	  sscanf(cmd, "%d", &port);
	  if(port == 0)
	    port = 2002;
	  session->remote_port = port;

	  memset(cmd, '\0', SANSON_SIZE_STRING);
	  fprintf(stdout, "protocol ?\n\t* IPv4 -> '4' (default)\n\t* IPv6 -> '6'\nchoice: ");
	  fflush(NULL);
	  fgets(cmd, sizeof(cmd), stdin);
	  sscanf(cmd, "%d", &proto);
	  if(proto == 0 || proto == 4)
	    session->proto = PROTO_IPV4;
	  else
	    session->proto = PROTO_IPV6;

	  memset(cmd, '\0', SANSON_SIZE_STRING);
	  fprintf(stdout, "ssl ?\n\t* SSL -> '1' (default)\n\t* None -> '2'\nchoice: ");
	  fflush(NULL);
	  fgets(cmd, sizeof(cmd), stdin);
	  sscanf(cmd, "%d", &ssl);
	  if(ssl == 0 || ssl == 1)
	    sanson_network_usessl(&session->sst);
	  else
	    sanson_network_notusessl(&session->sst);

	  fprintf(stdout, "-- remote address : %s:%d - mode : SOCKET_BIND - proto : IPv%d - ssl %d --\n", session->remote_ipv4, session->remote_port, (session->proto == PROTO_IPV4) ? 4 : 6, session->sst.ssl);
	  fprintf(stdout, "-- connecting to %s:%d --\n", session->remote_ipv4, session->remote_port);

	  if( connect_tcp(session) < 0 )
	    {
	      perror("connect_tcp()");
	      return;
	    }
	  fprintf(stdout, "-- connected to %s:%d --\n", session->remote_ipv4, session->remote_port);
	}

      sanson_network_send_cmd_list(&session->sst, NULL);
      get_paquet_list(&session->sst);
      
      /*
       * TODO: send guillotine
       */
    }
}
