#include "sanson.h"
#include "headman.h"

#ifdef ENABLE_READLINE
#include <readline/readline.h>
#include <readline/history.h>
#endif

#include <sys/ioctl.h>

void exec_init(void);
void exec_cleanup(void);
void exec_help(void);
void exec_run(headman_t *, char *);

pCommand mod =
{ "exec", NULL, SCOPE_SESSION, exec_init, exec_cleanup, exec_run, exec_help};

void exec_init(void)
{
#if __DEBUG_HEADMAN__
  fprintf(stdout, "-- EXEC INIT --\n");
#endif
}

void exec_cleanup(void)
{
#if __DEBUG_HEADMAN__
  fprintf(stdout, "-- EXEC CLEANUP --\n");
#endif
}

void exec_help(void)
{
  fprintf(stdout, "Execute a binary on the remote target.");
}

void exec_run(headman_t *hdm, char *argv)
{
  char cmd[SANSON_SIZE_STRING];
  char arg[SANSON_SIZE_STRING];
  char env[SANSON_SIZE_STRING];
  sanson_paquet_t *current_paquet;
  headman_session_t *session;
  char buffer[4096];
  int nb_lus;
  struct termios termios_stdin;
  struct termios termios_raw;
  struct termio temp_mode;
  fd_set set;
  char *prompt_cmd;
  char *prompt_tmp;

  if(hdm->current_session == -1)
    {
      fprintf(stdout, "-- your are not in a session --\n");
      return;
    }
  
  session = get_current_session(hdm);

#ifdef ENABLE_READLINE
  fprintf(stdout, "binary to execute: ");
  prompt_cmd = readline(NULL);
  prompt_tmp = prompt_cmd;
  while(isspace(*prompt_tmp)) ++prompt_tmp;
  while(*prompt_tmp != '\n' && *prompt_tmp != ' ' && *prompt_tmp != '\0') ++prompt_tmp;
  *prompt_tmp = '\0';
#else
  prompt_cmd = argv;
#endif
  
  for (current_paquet = session->sst.tailq_spt_t.tqh_first; current_paquet != NULL; current_paquet = current_paquet->entries.tqe_next)
    {
      if(strcmp(current_paquet->spht.name, prompt_cmd) == 0)
	{
	  memset(arg, '\0', sizeof(arg));
	  fprintf(stdout, "arguments: ");
	  fflush(NULL);
	  fgets(arg, sizeof(arg), stdin);
	  arg[strlen(arg) -1] = '\0';
	  
	  memset(env, '\0', sizeof(env));
	  fprintf(stdout, "environment variables: ");
	  fflush(NULL);
	  fgets(env, sizeof(env), stdin);
	  
	  fprintf(stdout, "-- preparing to run the binary '%s' --\n", prompt_cmd);

	  if( connect_tcp(session) < 0 )
	    {
	      perror("connect_tcp()");
	      return;
	    } 
	  if(session->sst.fd == -1)
	    {
	      fprintf(stdout, "-- your are not connected --\n");
	    }
	  else
	    {
	      sanson_network_send_cmd_exec(&session->sst, prompt_cmd);
	      sanson_network_send_argvenv(&session->sst, arg, env);

	      if ( ! isatty(1) )
		return;
	      
	      temp_mode.c_lflag=(ISIG|ICANON|ECHO|ECHOE);
	      temp_mode.c_iflag=(BRKINT|IGNPAR| /* ISTRIP| */ ICRNL|IXON);
	      temp_mode.c_oflag=(OPOST|ONLCR);
	      temp_mode.c_cflag=(CS7|PARENB|CREAD);
	      temp_mode.c_cc[VERASE]=('H'^64);
	      temp_mode.c_cc[VKILL]=('U'^64);
	      temp_mode.c_cc[VQUIT]=('\\'^64);
	      temp_mode.c_cc[VINTR]=('C'^64);
	      temp_mode.c_cc[VEOF]=('D'^64);
	      
	      if (ioctl(1, TCSETA, (char *) &temp_mode) < 0)
		return;
	      
	      tcgetattr(STDIN_FILENO, &termios_stdin);
	      tcgetattr(STDIN_FILENO, &termios_raw);
	      cfmakeraw(&termios_raw);
	      tcsetattr(STDIN_FILENO, TCSANOW, &termios_raw);
	      
	      while(1)
		{
		  FD_ZERO(&set);
		  FD_SET(session->sst.fd, &set);
		  FD_SET(STDIN_FILENO, &set);
		  
		  if (select(session->sst.fd+1, &set, NULL, NULL, NULL) < 0)
		    break;
		  
		  if (FD_ISSET(session->sst.fd, &set))
		    {
		      if ((nb_lus = read(session->sst.fd, buffer, 4096)) <= 0)
			break;
		      
		      write(STDOUT_FILENO, buffer, nb_lus);
		    }
		  
		  if (FD_ISSET(STDIN_FILENO, &set))
		    {
		      if ((nb_lus = read(STDIN_FILENO, buffer, 4096)) <= 0)
			break;
		      write(session->sst.fd, buffer, nb_lus);
		    }
		  
		  if (ioctl(1, TCSETA, (char *) &temp_mode) < 0)
		    return;
		}
	      
	      tcsetattr(STDIN_FILENO, TCSANOW, &termios_stdin);
	      
	      return;
	    }
	}
    }
  fprintf(stdout, "-- this binary does not exist --\n");
}
