#include "sanson.h"
#include "headman.h"

#ifdef ENABLE_READLINE
#include <readline/readline.h>
#include <readline/history.h>
#endif

void send_init(void);
void send_cleanup(void);
void send_help(void);
void send_run(headman_t *, char *);

pCommand mod =
{ "send", NULL, SCOPE_SESSION, send_init, send_cleanup, send_run, send_help};

void send_init(void)
{
#if __DEBUG_HEADMAN__
  fprintf(stdout, "-- SEND INIT --\n");
#endif
}

void send_cleanup(void)
{
#if __DEBUG_HEADMAN__
  fprintf(stdout, "-- SEND CLEANUP --\n");
#endif
}

void send_help(void)
{
  fprintf(stdout, "Send a binary to the remote host.");
}

void send_run(headman_t *hdm, char *argv)
{
  headman_session_t *session;
  sanson_paquet_t *spt;
  char cmd[SANSON_SIZE_STRING];
  int compress = 1;
  int crypt = 1;
  char *prompt_cmd;
  char *prompt_tmp;

  if(hdm->current_session == -1)
    {
      fprintf(stdout, "-- your are not in a session --\n");
      return;
    }

  session = get_current_session(hdm);

  
#ifdef ENABLE_READLINE
  fprintf(stdout, "file to send: ");
  prompt_cmd = readline(NULL);
  prompt_tmp = prompt_cmd;
  while(isspace(*prompt_tmp)) ++prompt_tmp;
  while(*prompt_tmp != '\n' && *prompt_tmp != ' ' && *prompt_tmp != '\0') ++prompt_tmp;
  *prompt_tmp = '\0';
#else
  prompt_cmd = argv;
#endif
  
  memset(cmd, '\0', SANSON_SIZE_STRING);
  fprintf(stdout, "compress ?\n\t* none\t-> '1' (default)\n\t* zip\t-> '2'\n\t* gzip\t-> '3'\nchoice: ");
  fflush(NULL);
  fgets(cmd, sizeof(cmd), stdin);
  sscanf(cmd, "%d", &compress);
  if(compress == 2)
    compress = SANSON_COMPRESS_ZIP;
  else if(compress == 3)
    compress = SANSON_COMPRESS_GZIP;
  else
    compress = SANSON_COMPRESS_NONE;
  
  memset(cmd, '\0', SANSON_SIZE_STRING);
  fprintf(stdout, "crypt ?\n\t* none\t-> '1' (default)\n\t* AES\t-> '2'\n\t* XOR\t-> '3'\nchoice: ");
  fflush(NULL);
  fgets(cmd, sizeof(cmd), stdin);
  sscanf(cmd, "%d", &crypt);
  if (2 == crypt)
    crypt = SANSON_CRYPT_AES;
  else if (3 == crypt)
    crypt = SANSON_CRYPT_XOR;
  else
    crypt = SANSON_CRYPT_NONE;
 
  if (sanson_make_paquet_binary(&session->sst, prompt_cmd, compress, crypt) < 0)
    {
      fprintf(stdout, "-- sanson_make_paquet_binary() error --\n");
      return;
    }
  
  sanson_session_paquet_name_view(&session->sst, prompt_cmd);
  
  sanson_paquet_t * tmp = sanson_session_paquet_name_get(&session->sst, prompt_cmd);

  if( connect_tcp(session) < 0 )
    {
      perror("connect_tcp()");
      return;
    } 

  if(session->sst.fd == -1)
    {
      fprintf(stdout, "-- your are not connected --\n");
    }
  else
    {
      sanson_network_send_cmd_send(&session->sst, prompt_cmd);
      sanson_network_send(&session->sst, &tmp->spht, sizeof(sanson_paquet_header_t), -1);
      sanson_network_send(&session->sst, tmp->binary, tmp->spht.size, -1);
    }
}
