#include "sanson.h"
#include "headman.h"

#include <sys/ioctl.h>

// bidouille d'un pseudo shell (pas de commandes native type cd...)

void shell_init(void);
void shell_cleanup(void);
void shell_help(void);
void shell_run(headman_t *, char *);

pCommand mod =
{ "shell", NULL, SCOPE_SESSION, shell_init, shell_cleanup, shell_run, shell_help};

void shell_init(void)
{
#if __DEBUG_HEADMAN__
  fprintf(stdout, "-- SHELL INIT --\n");
#endif
}

void shell_cleanup(void)
{
#if __DEBUG_HEADMAN__
  fprintf(stdout, "-- SHELL CLEANUP --\n");
#endif
}

void shell_help(void)
{
  fprintf(stdout, "Open a shell-like with Busybox.");
}


#define BUSYBOX_FILE "busybox"

void shell_run(headman_t *hdm, char *argv)
{
	char buffer[4096], arg[SANSON_SIZE_STRING];
	int nb_lus, i;
	headman_session_t *session;
	sanson_paquet_t *current_paquet;
	struct termios termios_stdin;
	struct termios termios_raw;
	struct termio temp_mode;
	fd_set set;
	
	if(hdm->current_session == -1)
	{
		fprintf(stdout, "-- your are not in a session --\n");
		return;
	}
  
	session = get_current_session(hdm);

	for (current_paquet = session->sst.tailq_spt_t.tqh_first; current_paquet != NULL; current_paquet = current_paquet->entries.tqe_next)
	{
		if(strcmp(current_paquet->spht.name, "busybox") == 0)
		{
			while(1)
			{
				fprintf(stdout, "$ ");
				fflush(NULL);
				fgets(arg, sizeof(arg), stdin);
				arg[strlen(arg) -1] = '\0';
				if(strcmp(arg, "exit") == 0) // Ctrl+D ?
					break;
				
				if(strlen(arg) != 0)
				{
  				sscanf(arg, "ash -c %s", arg); //// ! \\\\ -> malloc...

					if( connect_tcp(session) < 0 )
					  {
					    perror("connect_tcp()");
					    return;
					  } 
					
					if(session->sst.fd == -1)
					  {
					    fprintf(stdout, "-- your are not connected --\n");
					    return;
					  }
					sanson_network_send_cmd_exec(&session->sst, BUSYBOX_FILE); // doit etre deja present
					sanson_network_send_argvenv(&session->sst, arg, "");
					
					tcgetattr(STDIN_FILENO, &termios_stdin);
					tcgetattr(STDIN_FILENO, &termios_raw);
					cfmakeraw(&termios_raw);
					tcsetattr(STDIN_FILENO, TCSANOW, &termios_raw);
					
					// $ stty sane
					if ( ! isatty(1) )
						return;
	      
					temp_mode.c_lflag=(ISIG|ICANON|ECHO|ECHOE);
					temp_mode.c_iflag=(BRKINT|IGNPAR| ICRNL|IXON);
					temp_mode.c_oflag=(OPOST|ONLCR);
					temp_mode.c_cflag=(CS7|PARENB|CREAD);
					temp_mode.c_cc[VERASE]=('H'^64);
					temp_mode.c_cc[VKILL]=('U'^64);
					temp_mode.c_cc[VQUIT]=('\\'^64);
					temp_mode.c_cc[VINTR]=('C'^64);
					temp_mode.c_cc[VEOF]=('D'^64);
	      
					if (ioctl(1, TCSETA, (char *) &temp_mode) < 0)
						return;
	      
					tcgetattr(STDIN_FILENO, &termios_stdin);
					tcgetattr(STDIN_FILENO, &termios_raw);
					cfmakeraw(&termios_raw);
					tcsetattr(STDIN_FILENO, TCSANOW, &termios_raw);
					
					
					while(1)
					{
						FD_ZERO(&set);
						FD_SET(session->sst.fd, &set);
						FD_SET(STDIN_FILENO, &set);
					
						if (select(session->sst.fd+1, &set, NULL, NULL, NULL) < 0)
							break;
					
						if (FD_ISSET(session->sst.fd, &set))
						{
							if ((nb_lus = read(session->sst.fd, buffer, 4096)) <= 0)
								break;
							
							write(STDOUT_FILENO, buffer, nb_lus);
						}
					
						if (FD_ISSET(STDIN_FILENO, &set))
						{
							if ((nb_lus = read(STDIN_FILENO, buffer, 4096)) <= 0)
								break;
							write(session->sst.fd, buffer, nb_lus);
						}

						if (ioctl(1, TCSETA, (char *) &temp_mode) < 0)
						  return;
					}
					tcsetattr(STDIN_FILENO, TCSANOW, &termios_stdin);
					
				}
			}
			return;
		}
	}
	
	fprintf(stdout, "-- you need to upload the busybox binary first --\n");
}
