#include "netinet/in.h"

#include "sanson.h"
#include "headman.h"

#ifdef ENABLE_SSL

extern char ca_crt[];
extern char headman_crt[];
extern char headman_key[];

#define DEBUG_LEVEL 0

void my_debug( void *ctx, int level, char *str )
{
    if( level < DEBUG_LEVEL )
    {
        fprintf( (FILE *) ctx, "%s", str );
        fflush(  (FILE *) ctx  );
    }
}
#endif
int listen_tcp(headman_session_t *session, int proto, int port)	 
{	 
  struct sockaddr_in server;	 
  // struct sockaddr_in6 serverv6;	 
int fd;	 
 	 
  if(proto == PROTO_IPV4)	 
    {	 
      server.sin_family = AF_INET;	 
      server.sin_port = htons(port);	 
      server.sin_addr.s_addr = 0;	 
 	 
      if ((fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {	 
        perror("socket()");	 
        return -1;	 
      }	 
 	 
      session->local_ipv4 = "127.0.0.1";	 
      session->remote_ipv4 = "";	 
      session->local_port = port;	 
      session->remote_port = 0;	 
    }	 
  else /* PROTO_IPV6 */	 
    {	 
      /*	 
      server.sin6_family = AF_INET6;	 
      server.sin6_port = htons(port);	 
      server.sin6_addr.s_addr = 0;	 
 	 
      if ((fd = socket(AF_INET6, SOCK_STREAM, IPPROTO_TCP)) < 0) {	 
        perror("socket()");	 
        return -1;	 
      }	 
      */	 
    }	 
 	 
  if (bind(fd, (struct sockaddr *)&server, sizeof(struct sockaddr_in)) < 0) {	 
    perror("bind()");	 
    return -1;	 
  }	 
 	 
  if (listen(fd, 8) < 0) {	 
    perror("listen()");	 
    return -1;	 
  }	 
 	 
  session->sst.fd = fd;	 
 	 
  return 0;	 
}

int connect_tcp(headman_session_t *session)
{
  int ret;
  
  if(session->proto == PROTO_IPV4)
    {
      if (( ret = net_connect(&session->sst.fd, session->remote_ipv4, session->remote_port)) != 0)
	{
	  printf( " failed\n  ! net_connect returned %d\n\n", ret );
	  return -1;
	}

#ifdef ENABLE_SSL
      if (session->sst.ssl)
	{
	  sanson_network_init_client(&session->sst, ca_crt, headman_crt, headman_key);
	  
	  havege_init( &session->sst.hs );
	  
	  if( ( ret = ssl_init( &session->sst.ssl_ctx ) ) != 0 )
	    {
	      printf( " failed\n  ! ssl_init returned %d\n\n", ret );
	      return -1;
	    }
	  
	  ssl_set_endpoint( &session->sst.ssl_ctx, SSL_IS_CLIENT );
	  ssl_set_authmode( &session->sst.ssl_ctx, SSL_VERIFY_OPTIONAL );
	  
	  ssl_set_rng( &session->sst.ssl_ctx, havege_rand, &session->sst.hs );
	  ssl_set_dbg( &session->sst.ssl_ctx, my_debug, stdout );
	  ssl_set_bio( &session->sst.ssl_ctx, net_recv, &session->sst.fd,
		       net_send, &session->sst.fd );
	  
	  ssl_set_ciphers( &session->sst.ssl_ctx, ssl_default_ciphers );
	  ssl_set_session( &session->sst.ssl_ctx, 1, 600, &session->sst.ssn );
	  
	  ssl_set_ca_chain( &session->sst.ssl_ctx, &session->sst.cacert, SERVER_NAME );
	  ssl_set_own_cert( &session->sst.ssl_ctx, &session->sst.clicert, &session->sst.rsa );
	  
	  ssl_set_hostname( &session->sst.ssl_ctx, SERVER_NAME );
	  
	  while( ( ret = ssl_handshake( &session->sst.ssl_ctx ) ) != 0 )
	    {
	      if( ret != XYSSL_ERR_NET_TRY_AGAIN )
		{
		  printf( " failed\n  ! ssl_handshake returned %d\n\n", ret );
		  return -1;
		}
	    }
	  
	  printf( " ok\n    [ Cipher is %s ]\n",
		  ssl_get_cipher( &session->sst.ssl_ctx ) );
	  
	  printf( "  . Verifying peer X.509 certificate..." );
	  
	  if( ( ret = ssl_get_verify_result( &session->sst.ssl_ctx ) ) != 0 )
	    {
	      printf( " failed\n" );
	      
	      if( ( ret & BADCERT_EXPIRED ) != 0 )
		printf( "  ! server certificate has expired\n" );
	      
	      if( ( ret & BADCERT_REVOKED ) != 0 )
		printf( "  ! server certificate has been revoked\n" );
	      
	      if( ( ret & BADCERT_CN_MISMATCH ) != 0 )
		printf( "  ! CN mismatch (expected CN=%s)\n", SERVER_NAME );
	      
	      if( ( ret & BADCERT_NOT_TRUSTED ) != 0 )
		printf( "  ! self-signed or not signed by a trusted CA\n" );
	      
	      printf( "\n" );
	      return -1;
	    }
	  else
	    printf( " ok\n" );
	  
	  printf( "  . Peer certificate information    ...\n" );
	  printf( x509parse_cert_info( "      ", session->sst.ssl_ctx.peer_cert ) );
	}  
#endif
    }
  else /* PROTO_IPV6 */
    {
      /*
      client_v6.sin6_family = AF_INET6;
      client_v6.sin6_port = htons(session->remote_port);
      client_v6.sin6_addr.s6_addr = inet_addr(addr);
      
      if ((fd = socket(AF_INET6, SOCK_STREAM, IPPROTO_TCP)) < 0) {
	perror("socket()");
	return -1;
      }
      
      if (connect(fd, (struct sockaddr *)&client_v6, sizeof(struct sockaddr_in)) < 0)
	{
	  perror("connect");
	  return -1;
	  }
      */
    }

  return 0;
}
