#include "sanson.h"
#include "headman.h"

headman_session_t *get_current_session(headman_t *hdm)
{
  headman_session_t *current;

  TAILQ_FOREACH(current, &hdm->tailq_session_t, next) {
    if(hdm->current_session == current->id)
      return current;
  }

  return NULL;
}

void headman_session_init(headman_t *hdm, headman_session_t *new_session)
{
  sanson_session_init(&new_session->sst);
  sanson_network_init(&new_session->sst);

  new_session->sst.network = SANSON_SOCKET_CONNECTBACK;
  sanson_session_alloc_set(&new_session->sst, SANSON_ALLOC_MALLOC);
  new_session->id = hdm->next_new_session;

  new_session->proto = PROTO_IPV4;
  new_session->local_ipv4 = NULL;
  new_session->local_ipv6 = NULL;
  new_session->remote_ipv4 = NULL;
  new_session->remote_ipv6 = NULL;
  new_session->local_port = -1;
  new_session->remote_port = -1;

  hdm->current_session = new_session->id;
  hdm->next_new_session++;
  
  TAILQ_INSERT_TAIL(&hdm->tailq_session_t, new_session, next);
}

/*
 * get the binary list of the remote session
 */
int get_paquet_list(sanson_session_t *sst)
{
  sanson_paquet_t spt;
  
  memset(&spt, '\0', sizeof(sanson_paquet_t));
  sanson_network_recv(sst, &spt, sizeof(sanson_paquet_t), -1);
  
  while(spt.binary != 0)
    {
      sanson_paquet_view(&spt);
      sanson_network_recv(sst, &spt, sizeof(sanson_paquet_t), -1);
    }

  return 0;
}
