#include "sanson.h"
#include "headman.h"


#ifdef ENABLE_READLINE
#include <readline/readline.h>
#include <readline/history.h>
#endif

#include <signal.h>

typedef struct {
  char *name;			/* User printable name of the function. */
  //  rl_icpfunc_t *func;		/* Function to call to do the job. */
  char *doc;			/* Documentation for this function.  */
} COMMAND;

COMMAND commands[] = {
  { "cd", "Change to directory DIR" },
  { "delete", "Delete FILE" },
  { "help", "Display this text" },
  { "?", "Synonym for `help'" },
  { (char *)NULL, (char *)NULL }
};

extern char *xmalloc ();
char *dupstr(char *);
void sig_hand(int);
void init_sign();
char *command_generator( char *, int);
char **command_completion( char *, int, int);
void initialize_readline();

int parse_headman_shell(headman_t *hdm, char *cmd)
{
  char *first, *rest;

  XCMP(cmd, NULL, -1);

  if (*cmd == '\n')
      return 0;

  while(isspace(*cmd)) ++cmd;
  if (!*cmd)
    return 0;

  first = cmd;
  while(!isspace(*cmd)) ++cmd;
  *cmd = '\0';
  ++cmd;

  while(*cmd == ' ') ++cmd;
  rest = cmd;
  while(*cmd && *cmd != '\n') cmd++;
  *cmd = '\0';

  exec_command(hdm, first, rest);
  
  return 0;
}

char *print_prompt(headman_t *hdm)
{
  char *ret;

  ret = malloc(SANSON_SIZE_STRING);
  memset(ret, '\0', SANSON_SIZE_STRING);
  sprintf(ret, HEADMAN_PROMPT);

  if(hdm->current_session != -1)
    {
      sprintf(ret, "%s (session %d) ", ret, hdm->current_session);
    }
  sprintf(ret, "%s# ", ret);
  
  return ret;
}

void init_headman_shell(headman_t *hdm)
{
  int nb_commands;

  /* Licence */
  fprintf(stdout, "Sanson The Headman version");
  fprintf(stdout, "\033[32m");
  fprintf(stdout, " %s", SANSON_VERSION);
  fprintf(stdout, "\033[0m");
  fprintf(stdout, " codename ");
  fprintf(stdout, "\033[32m");
  fprintf(stdout, "%s\n", SANSON_CODENAME);
  fprintf(stdout, "\033[0m");

  fprintf(stdout, "Copyright (C) 2008 Sanson The Headman team.\n");
  fprintf(stdout, "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n");
  fprintf(stdout, "This is free software: you are free to change and redistribute it.\n");

  /* init of sessions */
  hdm->current_session = -1;
  hdm->next_new_session = 0;
  TAILQ_INIT(&hdm->tailq_session_t);

  /* init of commands */
  TAILQ_INIT(&hdm->tailq_command_t);
  nb_commands = load_commands(&hdm->tailq_command_t);
  fprintf(stdout, "-- %d commands loaded --\n", nb_commands);
}

void run_headman_shell(headman_t *hdm)
{
  char *cmd;

  init_sign(); /* init the signals */
  initialize_readline();

  while(1)
    {      
#ifdef ENABLE_READLINE
      cmd = readline(print_prompt(hdm));
#else
      cmd = malloc(SANSON_SIZE_STRING);
      memset(cmd, '\0', SANSON_SIZE_STRING);
      fprintf(stdout, "%s", print_prompt(hdm));
      fflush(NULL);
      fgets(cmd, SANSON_SIZE_STRING, stdin);
#endif
      parse_headman_shell(hdm, cmd);
      free(cmd);
    }
  
  //  unload_commands(&hdm->tailq_command_t);
}

void
sig_hand(int sign)
{
  switch(sign) {
  default:
    fprintf(stdout, "\n");
    break;
  }
}

#define SET_HAND(x,y) if ( -1 == sigaction(x,y,NULL) )	\
    {								    \
      perror("-- can't set the action for --\n");		    \
    }

void init_sign ( void )
{
  /* Signal Handling */
  struct sigaction new,old;
  
  memset(&new,0,sizeof(new));
  new.sa_handler = &sig_hand;
  new.sa_flags |= SIG_SETMASK;
  if ( -1 == sigfillset(&new.sa_mask) ) {
    perror( "-- can't fill the sigset --\n" );
    exit( EXIT_FAILURE );
  }
  if ( -1 == sigaction(SIGSEGV,&new,&old) ) {
    perror( "-- can't set the action for SIGSERV --\n" );
  }
  SET_HAND(SIGINT,&new);
  SET_HAND(SIGQUIT,&new);
  SET_HAND(SIGTERM,&new);
}

char * command_generator( char * text, int state )
{
  static int len;
  pCommand *current;
  char *name;

  if ( !state ) {
    len = strlen( text );
  }

  /*  
  current = tailq_command_t.tqh_first;

  while ( ( current != NULL) ) {
    name = strdup(current->name);//, strlen(current->name));

    if ( strncmp(name, text, len) == 0 ) {
      //      printf("%s ", name);
      return( dupstr(name) );
    }
    current = current->next.tqe_next;
    free(name);
  }
  */
  return( (char *)NULL );
}

char ** command_completion( char * text, int start, int end )
{
  char ** matches = NULL;

  if ( start == 0 ) {
    matches = rl_completion_matches( text, 
				     (rl_compentry_func_t *)command_generator );
  }
  
  return( matches );
}

void initialize_readline( void )
{
  rl_attempted_completion_function = 
    (CPPFunction *)command_completion;
}

char *dupstr(char *s)
{
  char *r;

  r = xmalloc (strlen (s) + 1);
  strcpy (r, s);
  return (r);
}
