#include "sanson.h"

void * sanson_alloc(sanson_session_t *sst, int size)
{
  struct rlimit tmp;
  void *addr;

  XCMP(sst, NULL, NULL);

  addr = NULL;

  switch (sst->alloc)
    {
    case SANSON_ALLOC_MMAP :
#if __DEBUG_ALLOC__
      sanson_debug("[%s:%s:%d] : ALLOC_MMAP %d",
                   __FILE__,
                   __FUNCTION__,
                   __LINE__,
		   size);
#endif
      XMMAP(addr, 
	    0,
	    size,
	    sst->alloc_flags,
	    sst->alloc_map,
	    -1, 
	    0,
	    NULL);
      break;

    case SANSON_ALLOC_MALLOC :
#if __DEBUG_ALLOC__
      sanson_debug("[%s:%s:%d] : ALLOC_MALLOC %d",
                   __FILE__,
                   __FUNCTION__,
                   __LINE__,
		   size);
#endif
      XMALLOC(addr, size, NULL);
      break;

    default :
      return NULL;
      break;
    }

#if __DEBUG_ALLOC__
  sanson_debug(" @ 0x%lx\n", addr);
#endif

  if (sst->swap)
    {
      getrlimit(RLIMIT_MEMLOCK, &tmp);
      if (tmp.rlim_cur < (sst->len_swap+size))
	{
	  tmp.rlim_cur = sst->len_swap+size+1;
	  tmp.rlim_max = sst->len_swap+size+1;
	  sst->len_swap += size;
	  sst->len_swap ++;
	  setrlimit(RLIMIT_MEMLOCK, &tmp);
	}

      XMLOCK((const void *)addr, size, NULL);
    }

  return addr;
}

int sanson_free(sanson_session_t *sst, void *addr, int size)
{
  XCMP(sst, NULL, -1);

  if (sst->swap)
    {
      XMUNLOCK((const void *)addr, size, -1);	      
    }

  switch (sst->alloc)
    {
    case SANSON_ALLOC_MMAP :
#if __DEBUG_ALLOC__
      sanson_debug("[%s:%s:%d] : FREE_MMAP %d",
                   __FILE__,
                   __FUNCTION__,
                   __LINE__,
		   size);
#endif
      XMUNMAP(addr, size, -1);
      break;
    case SANSON_ALLOC_MALLOC :
#if __DEBUG_ALLOC__
      sanson_debug("[%s:%s:%d] : FREE_MALLOC %d",
                   __FILE__,
                   __FUNCTION__,
                   __LINE__,
		   size);
#endif
      XFREE(addr, -1);
      break;
    default :
      return -1;
      break;
    }
#if __DEBUG_ALLOC__
  sanson_debug(" @ 0x%lx\n", addr);
#endif
  return 0;
}


int sanson_session_alloc_set(sanson_session_t *sst, int type)
{

  XCMP(sst, NULL, -1);

  if (type == SANSON_ALLOC_MMAP ||
      type == SANSON_ALLOC_MALLOC ||
      type == SANSON_ALLOC_TMPFS)
    {

      sst->alloc = type;
      return 0;
    }

  return -1;
}
