#include "sanson.h"

#include <asm/unistd.h>

#define ERESTARTSYS     512
#define ERESTARTNOINTR  513
#define ERESTARTNOHAND  514     /* restart if no handler.. */

#define PTRACE_SETOPTIONS       0x4200
#define PTRACE_GETEVENTMSG      0x4201
#define PTRACE_GETSIGINFO       0x4202
#define PTRACE_SETSIGINFO       0x4203
#define PTRACE_O_TRACESYSGOOD   0x00000001

void alloc();
void alloc_end();

void (*aalloc)() = alloc;

#define ALLOC_SIZE (alloc_end - alloc)

int sanson_ptrace_hexdump(sanson_session_t *sst, sanson_inject_t *sit, unsigned long start, int len)
{
  int i;
  char *dest;

  XCMP(sst, NULL, -1);
  XCMP(sit, NULL, -1);

  dest = sanson_alloc(sst, len);
  
  for (i = 0; i < len; i++)
    dest[i]=(char)ptrace(PTRACE_PEEKTEXT, sit->pid, start+i, 0);
  sanson_hexdump((unsigned char *)(dest), len, (int)(dest));

  sanson_free(sst, dest, len);

  return 0;
}

int sanson_ptrace_attach(sanson_session_t *sst, sanson_inject_t *sit)
{
  int res;

  XCMP(sst, NULL, -1);
  XCMP(sit, NULL, -1);

  fprintf(stdout, "attach @ %d\n", sit->pid);

  res = ptrace(PTRACE_ATTACH, sit->pid, NULL, NULL);
  if (res == -1) 
    {
      perror("ptrace attach :");
      return -1;
    }

  res = waitpid(sit->pid, NULL, WUNTRACED);
  if (res != sit->pid) 
    {
      perror("waitpid :");
      return -1;
    }

  res = ptrace(PTRACE_SETOPTIONS, sit->pid, NULL, PTRACE_O_TRACESYSGOOD);
  if (res == -1)
    {
      perror("ptrace setoptions :");
      return -1;
    }

  return 0;
}

int sanson_ptrace_detach(sanson_session_t *sst, sanson_inject_t *sit)
{
  int res;

  XCMP(sst, NULL, -1);
  XCMP(sit, NULL, -1);

  res = ptrace(PTRACE_DETACH, sit->pid, NULL, NULL);
  if (res == -1) 
    {
      perror("ptrace detach :");
      return -1;
    }
  fprintf(stdout, "detach @ %d\n", sit->pid);

  return 0;
}


unsigned long sanson_ptrace_alloc(sanson_session_t *sst, sanson_inject_t *sit, unsigned long tot_len)
{
  int res, i, s;
  unsigned long start, old_eip, llen, deb, fin, save_stack, new_alloc;
  char *len;
  void *buffer;

  XCMP(sst, NULL, -1);
  XCMP(sit, NULL, -1);

  new_alloc = 0;

  if (sst->alloc_getinj == SANSON_ALLOC_GETINJ_STACK)
    {
      ul_get_stack_maps(sit->pid, &deb, &fin);
      sanson_debug("STACK %d @ 0x%lx ---> 0x%lx\n", sit->pid, deb, fin);
      
      save_stack = ptrace(PTRACE_PEEKTEXT, sit->pid, deb, 0);
      printf("SAVE_STACK 0x%lx\n", save_stack);
      
      sanson_ptrace_hexdump(sst, sit, deb, 64);
    }

  res = ptrace(PTRACE_GETREGS, sit->pid, NULL, &sit->regs);
  if (res == -1)
    {
      perror("ptrace getregs :");
      return -1;
    }

  /*
  siginfo_t sig;
  res = ptrace(PTRACE_GETSIGINFO, sit->pid, NULL, &sig);
  printf("res %d\n", res);
  printf("%d %d\n", sig.si_signo, sig.si_code);
  if (sig.si_code & 0x80)
    printf("was in a syscall\n");

  printf("EAX 0x%lx %d ORIG_EAX 0x%lx\n", sit->regs.eax, (int)sit->regs.eax, sit->regs.orig_eax);
  */

  if (sit->regs.orig_eax == __NR_nanosleep)
    sit->regs.eax = 0;

  buffer = sanson_alloc(sst, ALLOC_SIZE);
  memcpy(buffer, aalloc, ALLOC_SIZE);

  sanson_hexdump(buffer, ALLOC_SIZE, (int)buffer);
  len = find_pattern(buffer, ALLOC_SIZE, "\x41\x41\x41\x41", strlen("\x41\x41\x41\x41"));

  sanson_debug("Length 0x41414141 ---> 0x%lx with 0x%lx\n", (unsigned long)len, tot_len);

  if (len != NULL)
    {
      llen = tot_len;
      *(len + 3) = ((llen) & 0xff000000) >> 24;
      *(len + 2) = ((llen) & 0x00ff0000) >> 16;
      *(len + 1) = ((llen) & 0x0000ff00) >> 8;
      *(len + 0) = ((llen) & 0x000000ff);
    }

  if (sst->alloc_getinj == SANSON_ALLOC_GETINJ_STACK)
    {
      len = find_pattern(buffer, ALLOC_SIZE, "\x42\x42\x42\x42", strlen("\x42\x42\x42\x42"));
      
      printf("Stack 0x42424242 ---> 0x%lx\n", (unsigned long)len);
      if (len != NULL)
	{
	  llen = deb;
	  *(len + 3) = ((llen) & 0xff000000) >> 24;
	  *(len + 2) = ((llen) & 0x00ff0000) >> 16;
	  *(len + 1) = ((llen) & 0x0000ff00) >> 8;
	  *(len + 0) = ((llen) & 0x000000ff);
	}
    }
  
  
  /*
  len = find_pattern(buffer, ALLOC_SIZE, "\x43\x43\x43\x43", strlen("\x43\x43\x43\x43"));

  printf("Stack 0x43434343 ---> 0x%lx\n", (unsigned long)len);
  if (len != NULL)
    {
      llen = sit->pid;
      *(len + 3) = ((llen) & 0xff000000) >> 24;
      *(len + 2) = ((llen) & 0x00ff0000) >> 16;
      *(len + 1) = ((llen) & 0x0000ff00) >> 8;
      *(len + 0) = ((llen) & 0x000000ff);
    }
  */
  
  sanson_hexdump(buffer, ALLOC_SIZE, (int)buffer);

  if (sst->alloc_inj == SANSON_ALLOC_INJ_STACK)
    {
      start = sit->regs.esp-512-ALLOC_SIZE;
      for (i=0; i < ALLOC_SIZE; i+=4) 
	{
	  res = ptrace(PTRACE_POKEDATA, sit->pid, start+i, *(int *)(buffer+i));
	  if (res == -1) 
	    {
	      perror("ptrace pokedata :");
	      return -1;
	    }
	}

      printf("REGS.EIP 0x%lx\n", sit->regs.eip);
      old_eip = sit->regs.eip;
      sit->regs.eip = start;
      if ((sit->regs.orig_eax >= 0) &&
	  (sit->regs.eax == -ERESTARTNOHAND ||
	   sit->regs.eax == -ERESTARTSYS ||
	   sit->regs.eax == -ERESTARTNOINTR)) 
	{
	  sit->regs.eip += 2; 
	  old_eip -= 2;
	}
      
      sit->regs.esp -= 4;
      res = ptrace(PTRACE_POKEDATA, sit->pid, sit->regs.esp, old_eip);
      if (res == -1) 
	{
	  perror("ptrace pokedata :");
	  return -1;
	}
    }

  res = ptrace(PTRACE_SETREGS, sit->pid, NULL, &sit->regs);
  if (res == -1)
    {
      perror("ptrace setregs :");
      return -1;
    }

  if (sst->alloc_getinj == SANSON_ALLOC_GETINJ_INT)
    {
      if((ptrace(PTRACE_CONT,sit->pid,NULL,NULL)) < 0)
	perror("ptrace_cont"), exit(-1);
      
      while (!WIFSTOPPED(s)) 
	waitpid(sit->pid, &s, WNOHANG);

      res = ptrace(PTRACE_GETREGS, sit->pid, NULL, &sit->regs);
      if (res == -1)
	{
	  perror("ptrace getregs :");
	  return -1;
	}

      new_alloc = sit->regs.eax;
    }
  
  // return new_pid_alloc = ptrace(PTRACE_PEEKTEXT, sit->pid, deb, 0);
  
  return new_alloc;
}


int sanson_inject(sanson_session_t *sst, sanson_paquet_t *spt, sanson_inject_t *sit)
{
  int res, i, j;
  unsigned long tot_len, new_pid_alloc;
  sanson_execution_t set;
  Elf32_Ehdr *hdr;
  Elf32_Phdr *phdr;
  Elf32_Shdr *section;

  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(sit, NULL, -1);

  memset(&set, '\0', sizeof(sanson_execution_t));
  
  userland_execve_feed_elf(spt, &set);
  
  hdr = (Elf32_Ehdr *) spt->binary;
  tot_len = 0;
  for (i = 0; i < hdr->e_phnum; i++)
    {
      phdr = spt->binary + hdr->e_phoff + i * hdr->e_phentsize;

      if(phdr->p_type == PT_LOAD)
	  tot_len += ALIGN(phdr->p_memsz, phdr->p_align);
    }
  section = spt->binary + hdr->e_shoff+set.idxgotplt*sizeof(Elf32_Shdr);
  tot_len += section->sh_size; /* Alloc the got ! */;

  sanson_debug("TOT_LEN 0x%lx %d\n", tot_len, (unsigned int)tot_len);

  sanson_ptrace_attach(sst, sit);

  new_pid_alloc = sanson_ptrace_alloc(sst, sit, tot_len);
  printf("ALLOC in %d @ 0x%lx\n", sit->pid, new_pid_alloc);

  for (i = 0; i < hdr->e_phnum; i++)
    {
      phdr = spt->binary + hdr->e_phoff + i * hdr->e_phentsize;
      
      if(phdr->p_type == PT_LOAD)
	{
	  for (j = 0; j < phdr->p_filesz; j +=4)
	    {
	      res = ptrace(PTRACE_POKEDATA, 
			   sit->pid,
			   new_pid_alloc+phdr->p_offset+j, 
			   *(int *)(spt->binary + phdr->p_offset + j));

	      if (res == -1) 
		{
		  perror("ptrace pokedata :");
		  return -1;
		}
	    }
	}
    }

  sanson_ptrace_hexdump(sst, sit, new_pid_alloc, 64);
  
  sanson_ptrace_detach(sst, sit);

  return 0;
}
