#define _GNU_SOURCE

#include "sanson.h"

int view_binary_maps(void)
{
  FILE *fd;
  int i;
  char buffer[500];
  char *tok, *s1;
  
  memset(buffer, '\0', sizeof(buffer));
  snprintf(buffer, sizeof(buffer), "/proc/%d/maps", getpid());

  sanson_debug("[%s:%s:%d] : Open %s\n", 
	       __FILE__,
	       __FUNCTION__,
	       __LINE__,
	       buffer);

  fd = fopen(buffer, "r");
  while(fgets(buffer, sizeof(buffer), fd) != NULL)
    {
      for (i = 0, tok = sanson_strtok_r(buffer, " ", &s1); 
	   tok; 
	   i++, tok = sanson_strtok_r(NULL, " ", &s1))
	{
	  switch (i)
	    {
	    case 0 :
	      fprintf(stdout, "%s ", tok);
	      break;
	    case 1 :
	      fprintf(stdout, "%s ", tok);
	      break;
	    case 2 :
	      fprintf(stdout, "%s ", tok);
	      break;
	    case 5 :
	      fprintf(stdout, "%s ", tok);
	      break;
	    }
	}
    }

  fclose(fd);

  return 0;
}

int add_virtual_target(virtual_target_t *vt, target_t *tt)
{
  XCMP(vt, NULL, -1);
  XCMP(tt, NULL, -1);

  memcpy(vt->buffer+(sizeof(target_t) * vt->nb), tt, sizeof(target_t));
  vt->lenc += sizeof(target_t);
  vt->nb++;

  return 0;
}

int view_virtual_target(virtual_target_t *vt)
{
  int i;
  target_t tt;

  XCMP(vt, NULL, -1);

  sanson_debug("[%s:%s:%d] : VIEW VIRTUAL TARGET %d\n",
	       __FILE__,
	       __FUNCTION__,
	       __LINE__,
	       vt->lenc);

  for (i = 0; i < vt->nb; i++)
    {
      memcpy(&tt, vt->buffer+(sizeof(target_t)*i), sizeof(target_t));    
      fprintf(stdout, "@ 0x%.8lx strlen(0x%.8lx) action[ %d ", tt.addr, tt.addrl, tt.action);
      switch(tt.action)
	{
	case LEON_ACTION_MMAP :
	  fprintf(stdout, "LEON_ACTION_MMAP]");
	  break;
	  
	case LEON_ACTION_MUNMAP :
	  fprintf(stdout, "LEON_ACTION_MUNMAP]");
	  break;

	case LEON_ACTION_ERASE :
	  fprintf(stdout, "LEON_ACTION_ERASE]");
	  break;
	  
	case LEON_ACTION_COPY :
	  fprintf(stdout, "LEON_ACTION_COPY]");
	  break;

	case LEON_ACTION_MPROTECT :
	  fprintf(stdout, "LEON_ACTION_MPROTECT]");
	  break;

	case LEON_ACTION_SIMPLECOPY :
	  fprintf(stdout, "LEON_ACTION_SIMPLECOPY]");
	  break;

	case LEON_ACTION_HEAP :
	  fprintf(stdout, "LEON_ACTION_HEAP]");
	  break;
	}

      fprintf(stdout, " flags %d orig @ 0x%.8lx strlen(0x%.8lx)\n", 
	      tt.flags, tt.orig, tt.origl);
    }
  return 0;
}

int ul_get_text_elf(sanson_session_t *sst, 
		    sanson_execution_t *set, 
		    unsigned long *begin, 
		    unsigned long *end)
{
  char buffer[sst->page_size*4];
  int c, id, data, phdrnum, i;
  unsigned long phdroff, base, basel;
  Elf32_Phdr phdr;

  XCMP(sst, NULL, -1);
  XCMP(set, NULL, -1);

  c = id = data = phdrnum = i = 0;
  phdroff = base = basel = 0;

  c = sizeof(buffer)-sizeof(Elf32_auxv_t);
  memcpy(buffer, (void *)(set->st_top-sizeof(buffer)), sizeof(buffer));
 
  while (c > 0)
    {
      memcpy (&id, &buffer[c], sizeof(id));
      memcpy (&data, &buffer[c+sizeof(data)],sizeof(data));
      if ((id == AT_PAGESZ)&&(data == sst->page_size)) 
	break;
    c--;
    }
 
  while (buffer[c] != AT_NULL && c < sizeof(buffer))
    {
      if (buffer[c]==AT_PHDR)
	{
	  memcpy (&phdroff, &buffer[c+sizeof(phdroff)], sizeof(phdroff));
	}
      if  (buffer[c]==AT_PHNUM)
	  memcpy (&phdrnum, &buffer[c+sizeof(phdrnum)], sizeof(phdrnum));

      c = c+sizeof(Elf32_auxv_t);
    } 
 

  if (phdroff == 0 || phdrnum==0) 
    return -1;
  
  sanson_debug("[%s:%s:%d] : AT_PHDR: 0x%lx  AT_PHNUM: 0x%x\n",
	       __FILE__,
	       __FUNCTION__,
	       __LINE__,
	       phdroff, 
	       phdrnum);

  for (i=0; i < phdrnum; i++)
      {
	memcpy(&phdr, (unsigned long *)(phdroff+(sizeof(Elf32_Phdr)*i)), sizeof(Elf32_Phdr));
	if (phdr.p_type != PT_LOAD)
	  continue;
	
       	sanson_debug("AT PT_LOAD MEMSZ %d 0x%lx FILESZ %d 0x%lx VADDR 0x%lx OFFSET 0x%lx FLAGS %d ALIGN 0x%lx\n", 
		     phdr.p_memsz, 
		     (unsigned long) phdr.p_memsz,
		     phdr.p_filesz, 
		     (unsigned long) phdr.p_filesz,
		     (unsigned long)phdr.p_vaddr,
		     (unsigned long)phdr.p_offset,
		     phdr.p_flags,
		     ALIGN(phdr.p_memsz, phdr.p_align));

	if (phdr.p_flags == (PF_X|PF_R))
	  {
	    *begin = phdr.p_vaddr;
	    *end = phdr.p_vaddr + ALIGN(phdr.p_memsz, phdr.p_align);
	    break;
	  }
      }
    
  return 0;
}

int ul_maps_down_at(sanson_session_t *sst, sanson_execution_t *set)
{
  target_t tt;
  char buffer[sst->page_size*4];
  int c, id, data, phdrnum, i;
  unsigned long phdroff, base, basel;
  Elf32_Phdr phdr;

  XCMP(sst, NULL, -1);
  XCMP(set, NULL, -1);

  c = id = data = phdrnum = i = 0;
  phdroff = base = basel = 0;

  c = sizeof(buffer)-sizeof(Elf32_auxv_t);
  memcpy(buffer, (void *)(set->st_top-sizeof(buffer)), sizeof(buffer));
 
  while (c > 0)
    {
      memcpy (&id, &buffer[c], sizeof(id));
      memcpy (&data, &buffer[c+sizeof(data)],sizeof(data));
      if ((id == AT_PAGESZ)&&(data == sst->page_size)) 
	break;
    c--;
    }
 
  while (buffer[c] != AT_NULL && c < sizeof(buffer))
    {
      if (buffer[c]==AT_PHDR)
	{
	  memcpy (&phdroff, &buffer[c+sizeof(phdroff)], sizeof(phdroff));
	}
      if  (buffer[c]==AT_PHNUM)
	  memcpy (&phdrnum, &buffer[c+sizeof(phdrnum)], sizeof(phdrnum));

      c = c+sizeof(Elf32_auxv_t);
    } 
 

  if (phdroff == 0 || phdrnum==0) 
    return -1;
  
  sanson_debug("[%s:%s:%d] : AT_PHDR: 0x%lx  AT_PHNUM: 0x%x\n",
	       __FILE__,
	       __FUNCTION__,
	       __LINE__,
	       phdroff, 
	       phdrnum);

  for (i=0; i < phdrnum; i++)
      {
	memcpy(&phdr, (unsigned long *)(phdroff+(sizeof(Elf32_Phdr)*i)), sizeof(Elf32_Phdr));
	if (phdr.p_type != PT_LOAD)
	  continue;
	
       	sanson_debug("AT PT_LOAD MEMSZ %d 0x%lx FILESZ %d 0x%lx VADDR 0x%lx OFFSET 0x%lx FLAGS %d ALIGN 0x%lx\n", 
		     phdr.p_memsz, 
		     (unsigned long) phdr.p_memsz,
		     phdr.p_filesz, 
		     (unsigned long) phdr.p_filesz,
		     (unsigned long)phdr.p_vaddr,
		     (unsigned long)phdr.p_offset,
		     phdr.p_flags,
		     ALIGN(phdr.p_memsz, phdr.p_align));

	if (phdr.p_flags == (PF_X|PF_R))
	  {
	    base = phdr.p_vaddr;
	    basel = ALIGN(phdr.p_memsz, phdr.p_align);
	    tt.addr = base;
	    tt.addrl = basel;
	    tt.flags = phdr.p_flags;
	    tt.action = LEON_ACTION_MUNMAP;
	    tt.orig = 0x0;
	    tt.origl = 0;
	    add_virtual_target(&set->vtt, &tt);  
	  }
	else
	  {
	    tt.addr = base+basel;
	    tt.addrl = ALIGN(phdr.p_memsz, phdr.p_align);
	    tt.flags = phdr.p_flags;
	    tt.action = LEON_ACTION_MUNMAP;
	    tt.orig = 0x0;
	    tt.origl = 0;
	    add_virtual_target(&set->vtt, &tt); 
	  }
      }
    
  return 0;
}

int ul_maps_down_procmaps(sanson_session_t *sst, sanson_execution_t *set)
{
  FILE *fd;
  int i;
  unsigned long offset;
  char buffer[1024];
  target_t tt;
  char *tok, *begin, *end, *s1, *s2;

  XCMP(sst, NULL, -1);
  XCMP(set, NULL, -1);

  memset(&tt, '\0', sizeof(target_t));
  memset(buffer, '\0', sizeof(buffer));

  snprintf(buffer, sizeof(buffer), "/proc/%d/maps", getpid());

  sanson_debug("[%s:%s:%d] : Open %s",
	       __FILE__,
	       __FUNCTION__,
	       __LINE__,
	       buffer);

  fd = fopen(buffer, "r");
  if (fd == NULL)
    return -1;

  offset = -1;

  /* offset != addresse de base jusqu'a la heap */
  while((fgets(buffer, sizeof(buffer), fd) != NULL) && (offset != tt.addr))
    {
      for (i = 0, tok = sanson_strtok_r(buffer, " ", &s1); 
	   tok; 
	   i++, tok = sanson_strtok_r(NULL, " ", &s1))
	{
	  switch (i)
	    {
	    case 0 :
	      begin = sanson_strtok_r(tok, "-", &s2);
	      end = sanson_strtok_r(NULL, "-", &s2);
	      
	      tt.addr = strtoul(begin, NULL, 16);
	      tt.addrl = strtoul(end, NULL, 16) - tt.addr;
	      break;

	    case 1 :
	      tt.flags = 0;
	      if (tok[0] == 'r')
		{
		  tt.flags = PROT_READ;
		}
	      if (tok[1] == 'w')
		{
		  tt.flags |= PROT_WRITE;
		}
	      if (tok[2] == 'x')
		{
		  tt.flags |= PROT_EXEC;
		}
	      break;
	    case 2 :
	      offset = strtoul(tok, NULL, 16);
	      break;
	    }
	}
      if (offset != tt.addr)
      	{
	  tt.orig = 0x0;
	  tt.origl = 0x0;
	  tt.action = LEON_ACTION_MUNMAP;
	  add_virtual_target(&set->vtt, &tt);
	}
    }
  fclose(fd);

  return 0;
}

unsigned long heap_proc_maps()
{
  FILE *fd;
  int i;
  char buffer[500];
  char *tok, *begin, *s1, *s2;
  unsigned long addr;

  memset(buffer, '\0', sizeof(buffer));
  snprintf(buffer, sizeof(buffer), "/proc/%d/maps", getpid());

  fd = fopen(buffer, "r");
  while(fgets(buffer, sizeof(buffer), fd) != NULL)
    {
      for (i = 0, tok = sanson_strtok_r(buffer, " ", &s1); 
	   tok; 
	   i++, tok = sanson_strtok_r(NULL, " ", &s1))
	{
	  switch (i)
	    {
	    case 0 :
	      begin = sanson_strtok_r(tok, "-", &s2);

	      addr = strtoul(begin, NULL, 16);
	      break;
	    case 5 :
	      if (!strcmp(tok, "[heap]\n"))
		{
		  fclose(fd);
		  return addr;
		}
	      break;
	    }
	}
    }
  fclose(fd);

  return 0;
}


int ul_get_stack_maps(int pid,
		      unsigned long *deb,
		      unsigned long *fin)
{
  FILE *fd;
  int i;
  char buffer[500];
  char *tok, *begin, *end, *s1, *s2;

  memset(buffer, '\0', sizeof(buffer));
  snprintf(buffer, sizeof(buffer), "/proc/%d/maps", pid);

  fd = fopen(buffer, "r");
  while(fgets(buffer, sizeof(buffer), fd) != NULL)
    {
      for (i = 0, tok = sanson_strtok_r(buffer, " ", &s1); 
	   tok; 
	   i++, tok = sanson_strtok_r(NULL, " ", &s1))
	{
	  switch (i)
	    {
	    case 0 :
	      begin = sanson_strtok_r(tok, "-", &s2);
	      end = sanson_strtok_r(NULL, "-", &s2);

	      break;
	    case 5 :
	      if (strstr(tok, "[stack]"))
		{
		  *deb = strtoul(begin, NULL, 16);
		  *fin = strtoul(end, NULL, 16);
		  fclose(fd);
		  return 0;
		}
	      break;
	    }
	}
    }
  fclose(fd);

  return -1;
}


int ul_get_libc_maps(sanson_session_t *sst, 
		     sanson_execution_t *set, 
		     unsigned long *deb,
		     unsigned long *fin)
{
  FILE *fd;
  int i;
  char buffer[500];
  char *tok, *begin, *end, *s1, *s2;

  memset(buffer, '\0', sizeof(buffer));
  snprintf(buffer, sizeof(buffer), "/proc/%d/maps", getpid());

  fd = fopen(buffer, "r");
  while(fgets(buffer, sizeof(buffer), fd) != NULL)
    {
      for (i = 0, tok = sanson_strtok_r(buffer, " ", &s1); 
	   tok; 
	   i++, tok = sanson_strtok_r(NULL, " ", &s1))
	{
	  switch (i)
	    {
	    case 0 :
	      begin = sanson_strtok_r(tok, "-", &s2);
	      end = sanson_strtok_r(NULL, "-", &s2);

	      break;
	    case 5 :
	      if (strstr(tok, "libc"))
		{
		  *deb = strtoul(begin, NULL, 16);
		  *fin = strtoul(end, NULL, 16);
		  fclose(fd);
		  return 0;
		}
	      break;
	    }
	}
    }
  fclose(fd);

  return -1;
}

int sanson_ul_get_libc(sanson_session_t *sst, 
		       sanson_execution_t *set, 
		       unsigned long *deb, 
		       unsigned long *fin)
{
  XCMP(sst, NULL, -1);
  XCMP(set, NULL, -1);
  XCMP(deb, NULL, -1);
  XCMP(fin, NULL, -1);

  switch (sst->get_libc)
    {
    case SANSON_UL_GET_LIBC_MAPS :
      ul_get_libc_maps(sst, set, deb, fin);
      break;
    case SANSON_UL_GET_LIBC_ELF :
      break;
    }

  return 0;
}

int sanson_ul_get_text(sanson_session_t *sst, 
		       sanson_execution_t *set, 
		       unsigned long *deb, 
		       unsigned long *fin)
{
  XCMP(sst, NULL, -1);
  XCMP(set, NULL, -1);
  XCMP(deb, NULL, -1);
  XCMP(fin, NULL, -1);

  switch (sst->get_text)
    {
    case SANSON_UL_GET_TEXT_MAPS :
      break;
    case SANSON_UL_GET_TEXT_ELF :
      ul_get_text_elf(sst, set, deb, fin);
      break;
    }
  return 0;
}

int sanson_ul_maps_down(sanson_session_t *sst, sanson_execution_t *set)
{
  XCMP(sst, NULL, -1);
  XCMP(set, NULL, -1);

  switch (sst->ul_maps_down)
    {
    case SANSON_UL_MAPS_DOWN_PROCMAPS :
      ul_maps_down_procmaps(sst, set);
      break;

    case SANSON_UL_MAPS_DOWN_ELF :
      ul_maps_down_at(sst, set);
      break;
    }

  return 0;
}

int ul_make_vt_new(sanson_session_t *sst, sanson_paquet_t *spt, virtual_target_t *vmaps)
{
  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(vmaps, NULL, -1);
  
/*
  int j;
  int match;

  for (i = 0, p = ptab; i < e->e_phnum; i++, p++)
    {
      if (p->p_type != PT_LOAD)
	continue;
      printf("SEARCH TEXT and DATA\n");

      match = 0;
      
      for (j = 0; j < vtt.nb; j++)
	{
	  memcpy(&tt, vtt.buffer+(sizeof(target_t)*j), sizeof(target_t));    
	  
	  if (p->p_flags == (PF_X|PF_R) && tt.flags == 5)
	    {
	      printf("SEARCH TEXT\n");

	      if (tt.addr == p->p_vaddr && tt.addr >= p->p_memsz)
		{
		  printf("MATCH TEXT\n");
		  tt.action = LEON_ACTION_COPY;
		  tt.orig = (unsigned long)spt->binary + p->p_offset;
		  tt.origl = p->p_memsz;
		  memcpy(vtt.buffer+(sizeof(target_t)*j), &tt, sizeof(target_t));
		  match = 1;
		}
	    }
	  else if (p->p_flags == (PF_R|PF_W) && tt.flags == 7) // DATA is rwx in guillotine ....
	    {
	      printf("SEARCH DATA\n");
	      if (tt.addr == p->p_vaddr && tt.addr >= p->p_memsz)
		{
		  printf("MATCH DATA\n");
		  tt.action = LEON_ACTION_COPY;
		  tt.orig = (unsigned long)spt->binary + p->p_offset;
		  tt.origl = p->p_memsz;
		  memcpy(vtt.buffer+(sizeof(target_t)*j), &tt, sizeof(target_t));
		  match = 1;
		}
	    }
	}
	  	 
      if (!match)
	{
	  printf("MATCH ANYTHING\n");
	  
	  ttmp.addr = p->p_vaddr;
	  ttmp.addrl = p->p_memsz;
	  ttmp.flags = 0;
	  if (p->p_flags & PF_X)
	    ttmp.flags |= PROT_EXEC;
	  if (p->p_flags & PF_R)
	    ttmp.flags |= PROT_READ;
	  if (p->p_flags & PF_W)
	    ttmp.flags |= PROT_WRITE;
	  ttmp.action = LEON_ACTION_MMAP;
	  ttmp.orig = (unsigned long)spt->binary + p->p_offset;
	  ttmp.origl = p->p_memsz;
	  add_virtual_target(&vtmp, &ttmp);
	}
    }
  */

  return 0;
}

int ul_make_vt_classic(sanson_session_t *sst, sanson_paquet_t *spt, virtual_target_t *vmaps)
{  
  Elf32_Ehdr *e = (Elf32_Ehdr *) spt->binary;
  Elf32_Phdr *ptab = (Elf32_Phdr *) ((char *)spt->binary + e->e_phoff);
  Elf32_Phdr *p;
  int i;
  unsigned long tot_len, addr_heap, len, load_addr, base;
  virtual_target_t vtmp;
  target_t tt, ttmp;

  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(vmaps, NULL, -1);

  tot_len = 0;
  memset(&vtmp, '\0', sizeof(virtual_target_t));

  /* Caculate the space */
  for (i = 0, p = ptab; i < e->e_phnum; i++, p++)
    {
      /* If it's not a loadable segment, skip it */
      if (p->p_type != PT_LOAD)
	continue;
      
      sanson_debug("PT_LOAD MEMSZ %d 0x%lx FILESZ %d 0x%lx VADDR 0x%lx OFFSET 0x%lx FLAGS %d\n", 
		   p->p_memsz, 
		   (unsigned long) p->p_memsz,
		   p->p_filesz, 
		   (unsigned long) p->p_filesz,
		   (unsigned long)p->p_vaddr,
		   (unsigned long)p->p_offset,
		   p->p_flags);
      
      tot_len += ALIGN(p->p_memsz, p->p_align);
    }

  for (i = 0, p = ptab; i < e->e_phnum; i++, p++)
    {
      if (p->p_type != PT_LOAD)
	continue;

      if (p->p_flags == (PF_X|PF_R))
	base = p->p_vaddr;
    }

  sanson_debug("TOT_LEN 0x%lx => %d\n", tot_len, (unsigned int)tot_len);


  addr_heap = heap_proc_maps();
  sanson_debug("HEAP @ 0x%lx\n", addr_heap);
  sanson_debug("BASE + TOT_LEN => 0x%lx\n", base + tot_len);
  /* Mmap all PT_LOAD */
  ttmp.addr = base;
  if (addr_heap > (base+tot_len))
    ttmp.addrl = tot_len + ((addr_heap - (base+tot_len)));
  else
    ttmp.addrl = tot_len + sst->page_size;

  ttmp.flags = PROT_READ|PROT_WRITE;
  ttmp.action = LEON_ACTION_MMAP;
  ttmp.orig = 0x0;
  ttmp.origl = MAP_PRIVATE|MAP_FIXED|MAP_ANONYMOUS;
  add_virtual_target(&vtmp, &ttmp);

  sanson_debug("MMAP de 0x%lx => 0x%lx (DIFF 0x%lx)\n", ttmp.addr, ttmp.addr+ttmp.addrl, ttmp.addrl);
  /* Copy the PT_LOAD */
  for (i = 0, p = ptab; i < e->e_phnum; i++, p++)
    {
      if (p->p_type != PT_LOAD)
	continue;
          
      ttmp.addr = p->p_vaddr;
      ttmp.addrl = p->p_memsz;
      ttmp.action = LEON_ACTION_SIMPLECOPY;
      ttmp.orig = (unsigned long)spt->binary + p->p_offset;
      ttmp.origl = p->p_filesz;
      add_virtual_target(&vtmp, &ttmp);
    }

  /* mprotect the section */
  for (i = 0, p = ptab; i < e->e_phnum; i++, p++)
    {
      if (p->p_type != PT_LOAD)
	continue;
      
      load_addr = base + ((p->p_vaddr-base)&~(p->p_align-1));
      len = ALIGN(p->p_memsz, p->p_align);

      ttmp.addr = load_addr;
      ttmp.addrl = len;
      ttmp.flags = 0;
      if (p->p_flags & PF_X)
	ttmp.flags |= PROT_EXEC;
      if (p->p_flags & PF_R)
	ttmp.flags |= PROT_READ;
      if (p->p_flags & PF_W)
	ttmp.flags |= PROT_WRITE;
      
      ttmp.action = LEON_ACTION_MPROTECT;
      ttmp.orig = 0x0;
      ttmp.origl = 0x0;
      add_virtual_target(&vtmp, &ttmp);
    }

  /* Copy in the right struct */
  for (i = 0; i < vtmp.nb; i++)
    {
      memcpy(&tt, vtmp.buffer+(sizeof(target_t)*i), sizeof(target_t));    
      add_virtual_target(vmaps, &tt);
    }

  /* Add the empty target */
  memset(&tt, '\0', sizeof(tt));
  add_virtual_target(vmaps, &tt);

  return 0;
}

int sanson_ul_make_vt(sanson_session_t *sst, sanson_paquet_t *spt, sanson_execution_t *set)
{
  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(set, NULL, -1);

  switch(sst->ul_make_vt)
    {
    case SANSON_UL_MAKE_VT_CLASSIC :
      ul_make_vt_classic(sst, spt, &set->vtt);
      break;

    case SANSON_UL_MAKE_VT_NEW :
      break;
    }

  return 0;
}
