#include "sanson.h"

unsigned int sanson_strspn( char *string,  char *charset)
{
   char *p, *q;

  for (q = string; *q != '\0'; ++q) {
    for (p = charset; *p != '\0' && *p != *q; ++p)
      ;
    if (*p == '\0')
      break;
  }
  return (q - string);
}

char * sanson_strpbrk( char *string,  char *brkset)
{
   char *p;

  do {
    for (p = brkset; *p != '\0' && *p != *string; ++p)
      ;
    if (*p != '\0')
      return ((char *)string);
  } while (*string++);

  return (NULL);
}

char * sanson_strtok_r(char *string,  char *sepset, char **lasts)
{
  char    *q, *r;

  /* first or subsequent call */
  if (string == ( char*)NULL)
    string = *lasts;

  if (string == ( char*)NULL)             /* return if no tokens remaining */
    return (NULL);

  q = string + sanson_strspn(string, sepset);    /* skip leading separators */

  if (*q == '\0')         /* return if no tokens remaining */
    return (NULL);

  if ((r = sanson_strpbrk(q, sepset)) == (char*)NULL)   /* move past token */
    *lasts = NULL;  /* indicate this is last token */
  else {
    *r = '\0';
    *lasts = r + 1;
  }
  return (q);
}


/* Rip from kernsh (GPL) */
void sanson_hexdump(unsigned char * data, 
		    unsigned int amount, 
		    size_t addr)
{
  unsigned int dp;
  unsigned int p;
  const char trans[] =
    "................................ !\"#$%&'()*+,-./0123456789"
    ":;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklm"
    "nopqrstuvwxyz{|}~...................................."
    "....................................................."
    "........................................";

  for ( dp = 1; dp <= amount; dp++ )
    {
      if ( (dp % 16) == 1 )
	{
	  fprintf( stdout, "%#08x | ", addr+dp-1 );
	}

      fprintf( stdout, "%02x ", data[dp-1] );
      if ( (dp % 8) == 0 && (dp % 16) != 0 )
	{
	  fputs( " ", stdout );
        }
      if ( (dp % 16) == 0 )
	{
	  fputs( "| ", stdout );
	  p = dp;
	  for ( dp -= 16; dp < p; dp++ )
	    {
	      fprintf( stdout, "%c", trans[data[dp]] );
            }
	  fputs( "\n", stdout );
        }
    }
  
  if ( (amount % 16) != 0 )
    {
      p = dp = 16 - ( amount % 16 );
      for ( dp = p; dp > 0; dp-- )
	{
	  fputs( "   ", stdout );
	  if ( ((dp % 8) == 0) && (p != 8) ) 
	    {
	      fputs( " ", stdout );
	    }
	}
      fputs( "| ", stdout );
      for ( dp = (amount - (16 - p)); dp < amount; dp++ ) {
	fprintf( stdout, "%c", trans[data[dp]] );
      }
    }
  
  fputs( "\n", stdout );
  return;
}

#if __DEBUG_SANSON__
void sanson_debug(const char *fmt,...)
{
  va_list args;

  va_start(args, fmt);
  
  vfprintf(stderr, fmt, args);
  fflush(stderr);
  
  va_end(args);
}
#endif

int sanson_hexdump_set(sanson_session_t *sst, int val)
{
  XCMP(sst, NULL, -1);

  sst->hexdump = val;

  return 0;
}

void *find_pattern(const void *haystack, int haystack_len,
		   const void *needle, int needle_len)
{
  const char *begin;
  const char *const last_possible
    = (const char *) haystack+ haystack_len - needle_len;

  if (needle_len == 0)
    return (void *) haystack;

  if (haystack_len < needle_len)
    return NULL;

  for (begin = (const char *) haystack; begin <= last_possible; ++begin)
    if (begin[0] == ((const char *) needle)[0] &&
	!memcmp ((const void *) &begin[1],
		 (const void *) ((const char *) needle + 1),
		 needle_len - 1))
      return (void *) begin;

  return NULL;
}
