#include "sanson.h"

int sanson_network_init(sanson_session_t *sst)
{
  XCMP(sst, NULL, -1);

  switch(sst->network)
    {
    case SANSON_SOCKET_BIND:
      break;

    case SANSON_SOCKET_CONNECTBACK :
      break;

    case SANSON_SOCKET_FIND :
      break;

    default :
      return -1;
      break;
    }

  memset( &sst->ssl_ctx, 0, sizeof( ssl_context ) );

  return 0;
}

int sanson_network_usessl(sanson_session_t *sst)
{
  sst->ssl = 1;

  return 0;
}

int sanson_network_notusessl(sanson_session_t *sst)
{
  sst->ssl = 0;

  return 0;
}

int sanson_network_init_server(sanson_session_t *sst, char *ca_crt, char *srv_crt, char *srv_key)
{
  int ret;

  if (sst->ssl)
    {
      memset( &sst->srvcert, 0, sizeof( x509_cert ) );
      
      ret = x509parse_crt( &sst->srvcert, (unsigned char *) srv_crt,
			   strlen( srv_crt ) );
      if( ret != 0 )
	{
	  printf( " failed\n  !  x509parse_crt returned %d\n\n", ret );
	  return -1;
	}
      
      ret = x509parse_crt( &sst->srvcert, (unsigned char *) ca_crt,
			   strlen( ca_crt ) );
      if( ret != 0 )
	{
	  printf( " failed\n  !  x509parse_crt returned %d\n\n", ret );
	  return -1;
	}

      ret =  x509parse_key( &sst->rsa, (unsigned char *) srv_key,
			    strlen( srv_key ), NULL, 0 );
      if( ret != 0 )
	{
	  printf( " failed\n  !  x509parse_key returned %d\n\n", ret );
	  return -1;
	}      
    }

 return 0;
}

int sanson_network_init_client(sanson_session_t *sst, char *ca_crt, char *cli_crt, char *cli_key)
{
  int ret;

  if (sst->ssl)
    {
      havege_init( &sst->hs );
      memset( &sst->ssn, 0, sizeof( ssl_session ) );

      memset( &sst->cacert, 0, sizeof( x509_cert ) );
      
      ret = x509parse_crt( &sst->cacert, (unsigned char *) ca_crt,
			   strlen( ca_crt ) );
      if( ret != 0 )
	{
	  printf( " failed\n  !  x509parse_crt returned %d\n\n", ret );
	  return -1;
	}
      
      memset( &sst->clicert, 0, sizeof( x509_cert ) );
      
      ret = x509parse_crt( &sst->clicert, (unsigned char *) cli_crt,
			   strlen( cli_crt ) );
      if( ret != 0 )
	{
	  printf( " failed\n  !  x509parse_crt returned %d\n\n", ret );
	  return -1;
	}
      
      ret = x509parse_key( &sst->rsa, (unsigned char *) cli_key,
			   strlen( cli_key ), NULL, 0 );
      if( ret != 0 )
	{
	  printf( " failed\n  !  x509parse_key returned %d\n\n", ret );
	  return -1;
	}
    }

  return 0;
}

int sanson_network_close(sanson_session_t *sst)
{
  XCMP(sst, NULL, -1);

  close(sst->fd);

  return 0;
}

int sanson_network_fd_set(sanson_session_t *sst, int fd)
{
  XCMP(sst, NULL, -1);

  sst->fd = fd;

  return 0;
}

int sanson_network_accept(sanson_session_t *sst)
{
  int new;

  XCMP(sst, NULL, -1);

  if ((new = accept(sst->fd, NULL, NULL)) == -1)
    {
      perror("accept()");
      exit(errno);
    }

  return new;
}

int sanson_network_get_fd(sanson_session_t *sst)
{
  XCMP(sst, NULL, -1);

  return sst->fd;
}

int sanson_network_get_realfd(sanson_session_t *sst)
{
  XCMP(sst, NULL, -1);
  
  if (sst->forced_fd != -1)
    return sst->forced_fd;

  return sst->fd;
}

int sanson_network_recv(sanson_session_t *sst, 
			void *buffer, 
			size_t size, 
			int flags)
{
  int fd, err, total, i;

  XCMP(sst, NULL, -1);
  XCMP(buffer, NULL, -1);

  sanson_debug("[%s:%s:%d] : RECV %d\n",
	       __FILE__,
	       __FUNCTION__,
	       __LINE__,
	       size);

  if (sst->ssl == 1)
    {
      
      total = 0;
      err = 0;
      for (i = 0;  i < size;  i += err)
        {
          err = ssl_read(&sst->ssl_ctx, buffer + i, size - i);
	  total += err;
	}
    }
  else
    {
      fd = sanson_network_get_realfd(sst);
      
      if (flags == -1)
	flags = MSG_WAITALL;
      
      XRECV(fd, buffer, size, flags, -1);
    }

  sanson_network_debug(sst, (unsigned char *)buffer, size, (size_t)buffer);
    

  return 0;
}

int sanson_network_send(sanson_session_t *sst, 
			void *buffer, 
			size_t size, 
			int flags)
{
  int i, err, total, fd;

  XCMP(sst, NULL, -1);
  XCMP(buffer, NULL, -1);

  sanson_debug("[%s:%s:%d] : SEND %d\n",
	       __FILE__,
	       __FUNCTION__,
	       __LINE__,
	       size);

  if (sst->ssl == 1)
    {
      total = 0;
      err = 0;
      for (i = 0;  i < size;  i += err)
        {
          err = ssl_write(&sst->ssl_ctx, buffer + i, size - i);
	  total += err;
	}
    }
  else
    {
      fd = sanson_network_get_realfd(sst);
      
      if (flags == -1)
	flags = 0;
      
      XSEND(fd, buffer, size, flags, -1);
    }

  sanson_network_debug(sst, (unsigned char *)buffer, size, (size_t)buffer);

  return 0;
}

int send_paquet_header(sanson_paquet_header_t *spht, int fd)
{
  XCMP(spht, NULL, -1);
  
  XSEND(fd, spht, sizeof(sanson_paquet_header_t), 0, -1);

  return 0;
}

int send_paquet(sanson_session_t *sst, char *name, int fd)
{
  sanson_paquet_t * tmp;

  XCMP(sst, NULL, -1);
  XCMP(name, NULL, -1);

  for (tmp = sst->tailq_spt_t.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next)
    {
      if (!strcmp(tmp->spht.name, name))
	break;
    }

  send_paquet_header(&tmp->spht, fd); 
  XSEND(fd, tmp->binary, tmp->spht.size, 0, -1);

  return 0;
}

void * recv_paquet(sanson_session_t *sst, char *name, int fd)
{
  sanson_paquet_header_t spht;
  void *map_next;
  
  XCMP(name, NULL, NULL);

  XSEND(fd, name, SANSON_SIZE_STRING_NAME, 0, NULL);

  memset(&spht, '\0', sizeof(sanson_paquet_header_t));
  XRECV(fd, &spht, sizeof(sanson_paquet_header_t), MSG_WAITALL, NULL);

  map_next = sanson_alloc(sst, spht.total);

  memcpy(map_next, &spht, sizeof(sanson_paquet_header_t));

  XRECV(fd, map_next+sizeof(sanson_paquet_header_t), spht.size, MSG_WAITALL, NULL);

  return map_next;
}

int sanson_network_send_cmd_send(sanson_session_t *sst, char *arg)
{
  char cmd[SANSON_SIZE_CMD];
  
  XCMP(sst, NULL, -1);

  memset(cmd, '\0', sizeof(cmd));
  
  if (arg != NULL)
    snprintf(cmd, sizeof(cmd), "%s %s", SANSON_CMD_SEND, arg);
  else
    snprintf(cmd, sizeof(cmd), "%s", SANSON_CMD_SEND);

  return sanson_network_send(sst, cmd, sizeof(cmd), -1);
}

int sanson_network_send_cmd_del(sanson_session_t *sst, char *arg)
{
  char cmd[SANSON_SIZE_CMD];
  
  XCMP(sst, NULL, -1);
  XCMP(arg, NULL, -1);

  memset(cmd, '\0', sizeof(cmd));
  
  snprintf(cmd, sizeof(cmd), "%s %s", SANSON_CMD_DEL, arg);

  return sanson_network_send(sst, cmd, sizeof(cmd), -1);
}

int sanson_network_send_cmd_list(sanson_session_t *sst, char *arg)
{
  char cmd[SANSON_SIZE_CMD];

  XCMP(sst, NULL, -1);

  memset(cmd, '\0', sizeof(cmd));

  if (arg != NULL)
    snprintf(cmd, sizeof(cmd), "%s %s", SANSON_CMD_LIST, arg);
  else
    snprintf(cmd, sizeof(cmd), "%s", SANSON_CMD_LIST);

  return sanson_network_send(sst, cmd, sizeof(cmd), -1);
}

int sanson_network_send_cmd_exit(sanson_session_t *sst, char *arg)
{
  char cmd[SANSON_SIZE_CMD];
  
  XCMP(sst, NULL, -1);

  memset(cmd, '\0', sizeof(cmd));
  
  if (arg != NULL)
    snprintf(cmd, sizeof(cmd), "%s %s", SANSON_CMD_EXIT, arg);
  else
    snprintf(cmd, sizeof(cmd), "%s", SANSON_CMD_EXIT);

  return sanson_network_send(sst, cmd, sizeof(cmd), -1);
}

int sanson_network_send_cmd_exec(sanson_session_t *sst, char *arg)
{
  char cmd[SANSON_SIZE_CMD];
  
  XCMP(sst, NULL, -1);
  XCMP(arg, NULL, -1);

  memset(cmd, '\0', sizeof(cmd));
  
  snprintf(cmd, sizeof(cmd), "%s %s", SANSON_CMD_EXEC, arg);

  return sanson_network_send(sst, cmd, sizeof(cmd), -1);
}

int sanson_network_send_argvenv(sanson_session_t *sst, char *arg, char *env)
{
  char cmd[SANSON_SIZE_EXEC];
  
  XCMP(sst, NULL, -1);

  memset(cmd, '\0', sizeof(cmd));
  
  if (arg != NULL && env != NULL)
    snprintf(cmd, sizeof(cmd), "%s%c%s", arg, '\0', env);
  else if (arg == NULL && env != NULL)
    snprintf(cmd, sizeof(cmd), "%c%s", '\0', env);
  else if (arg != NULL && env == NULL)
    snprintf(cmd, sizeof(cmd), "%s%c", arg, '\0');
  else
    snprintf(cmd, sizeof(cmd), "%c", '\0');
  
  return sanson_network_send(sst, cmd, sizeof(cmd), -1);
}

int sanson_network_send_cmd_option(sanson_session_t *sst, char *arg)
{
  char cmd[SANSON_SIZE_CMD];
  
  XCMP(sst, NULL, -1);
  XCMP(arg, NULL, -1);

  memset(cmd, '\0', sizeof(cmd));
  
  snprintf(cmd, sizeof(cmd), "%s %s", SANSON_CMD_OPTION, arg);

  return sanson_network_send(sst, cmd, sizeof(cmd), -1);
}

int sanson_network_send_cmd_killguillotine(sanson_session_t *sst, char *arg)
{
  char cmd[SANSON_SIZE_CMD];
  
  XCMP(sst, NULL, -1);

  memset(cmd, '\0', sizeof(cmd));
  
  if (arg != NULL)
    snprintf(cmd, sizeof(cmd), "%s %s", SANSON_CMD_KILLGUILLOTINE, arg);
  else
    snprintf(cmd, sizeof(cmd), "%s", SANSON_CMD_KILLGUILLOTINE);

  return sanson_network_send(sst, cmd, sizeof(cmd), -1);
}

#if __DEBUG_NETWORK__
int sanson_network_debug(sanson_session_t *sst, 
			  unsigned char * data, 
			  unsigned int amount, 
			  size_t addr)
{
  int size;

  XCMP(sst, NULL, -1);

  size = amount;

  if (sst->hexdump != -1)
    size = sst->hexdump;

  sanson_hexdump(data, size, addr);

  return 0;
}
#endif
