#include "sanson.h"

int paquet_crypt(sanson_session_t *sst,
		 char *bufclair, 
		 size_t len,
		 char *bufcrypt, 
		 size_t lenc)
{
  char *pbufcrypt;
  char *pbufclair;

  unsigned char IV[16];
  unsigned char digest[32];
  unsigned char buffer[1024];
  int i, lastn, diff, n;
  off_t offset;

  aes_context aes_ctx;
  sha2_context sha_ctx;

  memset(bufcrypt, '\0', lenc);
  //fprintf(stdout, "BUFCRYPT @ 0x%lx %d BUFCLAIR @ 0x%lx %d\n", (unsigned long)bufcrypt, lenc, (unsigned long)bufclair, len);

  for( i = 0; i < 8; i++ )
    buffer[i] = (unsigned char)( len >> ( i << 3 ) );
  
  sha2_starts( &sha_ctx, 0 );
  sha2_update( &sha_ctx, buffer, 8 );
  sha2_finish( &sha_ctx, digest );
  
  memcpy( IV, digest, 16 );

  lastn = (int)( len & 0x0F );
  
  IV[15] = (unsigned char)
    ( ( IV[15] & 0xF0 ) | lastn );

  pbufclair = bufclair;
  pbufcrypt = bufcrypt;
  memcpy(bufcrypt, IV, 16);

  pbufcrypt += 16;

  memset( digest, 0,  32 );
  memcpy( digest, IV, 16 );
  
  for( i = 0; i < 8192; i++ )
    {
      sha2_starts( &sha_ctx, 0 );
      sha2_update( &sha_ctx, digest, 32 );
      sha2_update( &sha_ctx, sst->key, sst->keylen );
      sha2_finish( &sha_ctx, digest );
    }

  aes_setkey_enc( &aes_ctx, digest, 256 );
  sha2_hmac_starts( &sha_ctx, digest, 32, 0 );

  for( offset = 0; offset < len; offset += 16 )
    {
      n = ( len - offset > 16 ) ? 16 : (int)
	( len - offset );
      
      memcpy(buffer, pbufclair, n);
      pbufclair += n;

      for( i = 0; i < 16; i++ )
	buffer[i] = (unsigned char)( buffer[i] ^ IV[i] );

      
      aes_crypt_ecb( &aes_ctx, AES_ENCRYPT, buffer, buffer );
      sha2_hmac_update( &sha_ctx, buffer, 16 );

      memcpy(pbufcrypt, buffer, 16);
      pbufcrypt += 16;
      memcpy( IV, buffer, 16 );
    }

  sha2_hmac_finish( &sha_ctx, digest );
  memcpy(pbufcrypt, digest, 32);

  pbufcrypt += 32;
  diff = (int)pbufcrypt - (int)bufcrypt;

  //  printf("DIFF WRITE %d\n", diff);
  //printf("BUFFER CHIFFRE\n");

  memset( buffer, 0, sizeof( buffer ) );
  memset( digest, 0, sizeof( digest ) ); 
  memset( &aes_ctx, 0, sizeof(  aes_context ) );
  memset( &sha_ctx, 0, sizeof( sha2_context ) );

  return diff;
}

int paquet_uncrypt(sanson_session_t *sst,
		   char *bufcrypt, 
		   size_t len, 
		   char *bufuncrypt, 
		   size_t lenc)
{
  char *pbufcrypt;
  char *pbufuncrypt;
    
  unsigned char IV[16];
  unsigned char digest[32];
  unsigned char buffer[1024];
  unsigned char tmp[16];
  int i, lastn, ret, n;
  off_t offset;

  aes_context aes_ctx;
  sha2_context sha_ctx;
 
  //printf("UNCRYPT WITH KEY %s %d\n", sst->key, sst->keylen);

  //printf("BUFUNCRYPT @ 0x%lx %d BUFCRYPT @ 0x%lx %d\n", (unsigned long)bufuncrypt, lenc, (unsigned long)bufcrypt, len);

  pbufcrypt = bufcrypt;
  pbufuncrypt = bufuncrypt;

  len -= ( 16 + 32 );
  
  memcpy(buffer, pbufcrypt, 16);
  pbufcrypt += 16;
  memcpy( IV, buffer, 16 );
  lastn = IV[15] & 0x0F;

  memset( digest, 0,  32 );
  memcpy( digest, IV, 16 );
  
  for( i = 0; i < 8192; i++ )
    {
      sha2_starts( &sha_ctx, 0 );
      sha2_update( &sha_ctx, digest, 32 );
      sha2_update( &sha_ctx, sst->key, sst->keylen );
      sha2_finish( &sha_ctx, digest );
    }

  memset( sst->key, '\0', sizeof( sst->key ) );
  sst->keylen = 0;

  aes_setkey_dec( &aes_ctx, digest, 256 );
  sha2_hmac_starts( &sha_ctx, digest, 32, 0 );

  for( offset = 0; offset < len; offset += 16 )
    {
      memcpy(buffer, pbufcrypt, 16);
      pbufcrypt += 16;

      memcpy( tmp, buffer, 16 );
      
      sha2_hmac_update( &sha_ctx, buffer, 16 );
      aes_crypt_ecb( &aes_ctx, AES_DECRYPT, buffer, buffer );

      for( i = 0; i < 16; i++ ) 
	buffer[i] = (unsigned char)( buffer[i] ^ IV[i] );
      
      memcpy( IV, tmp, 16 );
      
      n = ( lastn > 0 && offset == len - 16 )
	? lastn : 16;

      memcpy(pbufuncrypt, buffer, n);
      pbufuncrypt += n;
    }

  ret = 0;

  sha2_hmac_finish( &sha_ctx, digest );
  
  memcpy(buffer, pbufcrypt, 32);

  if( memcmp( digest, buffer, 32 ) != 0 )
    {
      fprintf(stdout, "HMAC check failed: wrong key, or file corrupted.\n" );
      ret =  -1;
    }

  memset( buffer, 0, sizeof( buffer ) );
  memset( digest, 0, sizeof( digest ) ); 
  memset( &aes_ctx, 0, sizeof(  aes_context ) );
  memset( &sha_ctx, 0, sizeof( sha2_context ) );

  return 0;
}

int make_paquet_crypt(sanson_session_t *sst, sanson_paquet_t *spt, int fd)
{
  int diff;
  struct stat st;
  void *tmp;

  XSEEK(fd, 0, SEEK_SET, -1);

  XFSTAT(fd, &st, -1);

  if (spt->spht.crypt == SANSON_CRYPT_NONE)
    {
      spt->binary = sanson_alloc(sst, st.st_size);
      
      XREAD(fd, spt->binary, st.st_size, -1);
      
      spt->spht.size = st.st_size;
      spt->spht.nb = -1;
      
      spt->spht.total = sizeof(sanson_paquet_header_t) + spt->spht.size; 
    }

  if (spt->spht.crypt == SANSON_CRYPT_AES)
    {
      fprintf(stdout, "Enter key : ");
      fgets((char *)sst->key, sizeof(sst->key), stdin);
      sst->key[strlen((const char *)sst->key) - 1] = '\0';

      sst->keylen = strlen((const char *)sst->key);

      //printf("KEY %s %d\n", sst->key, sst->keylen);

      spt->binary = sanson_alloc(sst, st.st_size);
      XREAD(fd, spt->binary, st.st_size, -1);
      
      tmp = sanson_alloc(sst, st.st_size+100);
      memset(tmp, '\0', st.st_size+100);

      diff = paquet_crypt(sst, spt->binary, st.st_size, tmp, st.st_size+100);

      sanson_free(sst, spt->binary, st.st_size);
      spt->binary = tmp;

      spt->spht.size = diff;
      spt->spht.nb = -1;
      
      spt->spht.total = sizeof(sanson_paquet_header_t) + spt->spht.size; 
    }

  return 0;
}

int make_paquet_binary_static(sanson_session_t *sst, sanson_paquet_t *spt, int fd)
{
  spt->spht.type = SANSON_TYPE_STATIC;

  make_paquet_crypt(sst, spt, fd);

  return 0;
}

int make_paquet_binary_dyn(sanson_session_t *sst, sanson_paquet_t *spt, int fd)
{
  spt->spht.type = SANSON_TYPE_DYN;

  make_paquet_crypt(sst, spt, fd);

  return 0;
}

int sanson_make_paquet_binary(sanson_session_t *sst, char *fname, int compress, int crypt)
{
  int fd, i, dyn;
  Elf32_Ehdr e;
  Elf32_Phdr p;

  sanson_paquet_t *spt;

  XCMP(sst, NULL, -1);
  XCMP(fname, NULL, -1);

  XOPEN(fd, fname, O_RDONLY, 0777, -1);
  XREAD(fd, &e, sizeof(e), -1);
  
  switch (e.e_type)
    {
    case ET_EXEC :
      sanson_debug("[%s:%s:%d] : ET_EXEC\n", 
		   __FILE__, 
		   __FUNCTION__,
		   __LINE__);

      spt = sanson_session_paquet_new(sst, fname);
      spt->spht.compress = compress;
      spt->spht.crypt = crypt;

      sanson_debug("[%s:%s:%d] : STATIC\n", 
		   __FILE__, 
		   __FUNCTION__,
		   __LINE__);
      
      make_paquet_binary_static(sst, spt, fd);
      
      break;

    case ET_DYN :
      sanson_debug("[%s:%s:%d] : ET_DYN\n", 
		   __FILE__, 
		   __FUNCTION__, 
		   __LINE__);
     
      spt = sanson_session_paquet_new(sst, fname);
      spt->spht.compress = compress;
      spt->spht.crypt = crypt;

      dyn = 0;
      for (i = 0; i<e.e_phnum; i++)
	{
	  XSEEK(fd, e.e_phoff+(sizeof(Elf32_Phdr) * i), SEEK_SET, -1);
 
	  XREAD(fd, &p, sizeof(p), -1);

	  if (p.p_type == PT_DYNAMIC)
	    {
	      dyn++;
	    }
	}

      if (dyn)
	{
	  sanson_debug("[%s:%s:%d] : DYNAMIC\n", 
		       __FILE__, 
		       __FUNCTION__,
		       __LINE__);

	  make_paquet_binary_dyn(sst, spt, fd);
	}
      
      break;

    default :
      sanson_debug("[%s:%s:%d] : Oops i did it again !!\n", 
		   __FILE__, 
		   __FUNCTION__, 
		   __LINE__);
      break;
    }

  return 0;
}

int sanson_make_virtual_paquet_binary(sanson_session_t *sst, 
				      char *name, 
				      int compress, 
				      int crypt, 
				      int type,
				      unsigned long len,
				      void *ptr)
{
  sanson_paquet_t *spt;

  spt = sanson_session_paquet_new(sst, name);
  if (spt == NULL)
    return -1;

  spt->spht.compress = compress;
  spt->spht.crypt = crypt;

  spt->spht.type = type;
    
  spt->binary = ptr;

  spt->spht.size = len;

  spt->spht.nb = -1;
  spt->spht.total = sizeof(sanson_paquet_header_t) + spt->spht.size;

  return 0;
}
