#include "sanson.h"

int sanson_session_init(sanson_session_t *sst)
{
  XCMP(sst, NULL, -1);

  sanson_session_alloc_set(sst, SANSON_ALLOC_MMAP);

  sst->fd		= -1;
  sst->forced_fd	= -1;

  sst->hexdump		= -1;

  sst->network		= SANSON_SOCKET_BIND;
  sst->random		= SANSON_RANDOM_BRUTEFORCE;
  sst->ul_maps_down	= SANSON_UL_MAPS_DOWN_PROCMAPS;
  sst->ul_make_vt	= SANSON_UL_MAKE_VT_CLASSIC;
  sst->ul_exec_dyn	= SANSON_UL_EXEC_DYN_QUICKANDDIRTY;
  sst->get_libc		= SANSON_UL_GET_LIBC_MAPS;
  sst->get_text		= SANSON_UL_GET_TEXT_ELF;
  sst->find_name	= SANSON_FIND_NAME_STACK;
  sst->alloc_inj	= SANSON_ALLOC_INJ_STACK;
  sst->alloc_getinj	= SANSON_ALLOC_GETINJ_INT;
  sst->page_size	= SANSON_PAGE_SIZE_DEFAULT;

  sst->alloc_flags	= SANSON_ALLOC_PROT_DEFAULT;
  sst->alloc_map	= SANSON_ALLOC_MAP_DEFAULT;

  sst->eat_stack	= SANSON_EAT_STACK_DEFAULT;

  sst->swap		= 0;
  sst->len_swap		= 0;

  memset(sst->key, '\0', sizeof(sst->key));
  sst->keylen		= 0;

  sst->ssl		= 0;

  TAILQ_INIT(&sst->tailq_spt_t);  
  
  return 0;
}

int sanson_session_paquet_add(sanson_session_t *sst, sanson_paquet_t *spt)
{
  sanson_paquet_t *tmp;

  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);

  for (tmp = sst->tailq_spt_t.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next)
    {
      if (!strcmp(tmp->spht.name, spt->spht.name))
	return -1;
    }

  TAILQ_INSERT_TAIL(&sst->tailq_spt_t, spt, entries);

  return 0;
}

int sanson_session_paquet_del(sanson_session_t *sst, char *name)
{
  int size;
  sanson_paquet_t *tmp;

  XCMP(sst, NULL, -1);
  XCMP(name, NULL, -1);

  for (tmp = sst->tailq_spt_t.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next)
    {
      if (!strcmp(tmp->spht.name, name))
	{
	  TAILQ_REMOVE(&sst->tailq_spt_t, tmp, entries);
	  /* Erase binary !!! */
	  memset(tmp->binary, '\0', tmp->spht.size);
	  sanson_free(sst, tmp->binary, tmp->spht.size);

	  /* Erase Lib !!! */
	  /* FIXME */

	  /* Save size */
	  size = tmp->spht.total;
	  /* Erase header */
	  memset(&tmp->spht, '\0', sizeof(sanson_paquet_header_t));
	  return sanson_free(sst, tmp, size);
	}
    }

  return -1; /* no paquet found */
}

sanson_paquet_t *sanson_session_paquet_new(sanson_session_t *sst, char *fname)
{
  sanson_paquet_t *tmp;
  XCMP(sst, NULL, NULL);

  tmp = sanson_alloc(sst, sizeof(sanson_paquet_t));

  memset(tmp, '\0', sizeof(sanson_paquet_t));
  memset(&tmp->spht, '\0', sizeof(sanson_paquet_header_t));
  snprintf(tmp->spht.name, sizeof(tmp->spht.name), "%s", fname);
      
  tmp->binary = NULL;
  tmp->libs = NULL;
  tmp->spht.crypt = SANSON_CRYPT_NONE;
  tmp->spht.compress = SANSON_COMPRESS_NONE;

  TAILQ_INSERT_TAIL(&sst->tailq_spt_t, tmp, entries);

  return tmp;
}

int sanson_session_free(sanson_session_t *sst)
{
  sanson_paquet_t *spt, *next;

  XCMP(sst, NULL, -1);


  for (spt = sst->tailq_spt_t.tqh_first; spt != NULL; spt = next)
    {
      next = spt->entries.tqe_next;
      TAILQ_REMOVE(&sst->tailq_spt_t, spt, entries);

      sanson_free(sst, spt->binary, spt->spht.size);

      
      sanson_free(sst, spt, sizeof(sanson_paquet_t));
    }

  return 0;
}

int sanson_session_free_without_paquet(sanson_session_t *sst, sanson_paquet_t *wspt)
{
  sanson_paquet_t *spt, *next;

  XCMP(sst, NULL, -1);
  XCMP(wspt, NULL, -1);

  for (spt = sst->tailq_spt_t.tqh_first; spt != NULL; spt = next)
    {
      next = spt->entries.tqe_next;
      if (spt != wspt)
	{
	  TAILQ_REMOVE(&sst->tailq_spt_t, spt, entries);
	  
	  sanson_free(sst, spt->binary, spt->spht.size);

	  sanson_free(sst, spt, sizeof(sanson_paquet_t));
	}
    }

  return 0;
}

int sanson_paquet_header_view(sanson_paquet_header_t * spht)
{
  XCMP(spht, NULL, -1);

  fprintf(stdout, "[+] NAME : %s\n", spht->name);
  fprintf(stdout, "\t [+] TOTAL SIZE %d\n", (unsigned int) spht->total);
  fprintf(stdout, "\t [+] TYPE %d\n", spht->type);
  fprintf(stdout, "\t [+] COMPRESS %d\n", spht->compress);
  fprintf(stdout, "\t [+] CRYPT %d\n", spht->crypt);
  fprintf(stdout, "\t [+] SIZE %d\n", (unsigned int)spht->size);
  fprintf(stdout, "\t [+] NB %d\n", spht->nb);

  return 0;
}

int sanson_paquet_view(sanson_paquet_t * spt)
{
  XCMP(spt, NULL, -1);

  fprintf(stdout, "[+] Header :\n");
  sanson_paquet_header_view(&spt->spht);
  fprintf(stdout, "[+] Binary : 0x%lx\n", (unsigned long)spt->binary);
  fprintf(stdout, "[+] Libs : 0x%lx\n", (unsigned long)spt->libs);
  
  return 0;
}

int sanson_session_paquet_name_view(sanson_session_t * sst, char * name)
{
  sanson_paquet_t * tmp;

  XCMP(sst, NULL, -1);
  XCMP(name, NULL, -1);


  for (tmp = sst->tailq_spt_t.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next)
    {
      if (!strcmp(tmp->spht.name, name))
	{
	  return sanson_paquet_view(tmp); 
	}
    }
  
  return -1;
}


sanson_paquet_t * sanson_session_paquet_name_get(sanson_session_t * sst, char * name)
{
  sanson_paquet_t * tmp;

  XCMP(sst, NULL, NULL);
  XCMP(name, NULL, NULL);

  for (tmp = sst->tailq_spt_t.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next)
    {
      if (!strcmp(tmp->spht.name, name))
	{
	  return tmp; 
	}
    }
  
  return NULL;
}
