#include "sanson.h"
#include "leon.h"

void leon();
void leon_end();

void (*lleon)() = leon;

#define LEON_SIZE (leon_end - leon)

int userland_execve_paquet_uncrypt(sanson_session_t *sst,
				   sanson_paquet_t *spt)
{
  void *tmp;

  if (spt->spht.crypt == SANSON_CRYPT_AES)
    {
      tmp = sanson_alloc(sst, spt->spht.size);
      memset(tmp, '\0', spt->spht.size);

      if (paquet_uncrypt(sst, spt->binary, spt->spht.size, tmp, spt->spht.size) < 0)
	{
	  sanson_free(sst, spt->binary, spt->spht.size);
	  return -1;
	}

      sanson_free(sst, spt->binary, spt->spht.size);

      spt->binary = tmp;
    }

  return 0;
}

int sanson_userland_execve(sanson_session_t *sst,
			   sanson_paquet_t *spt,
			   sanson_arg_stack_t *sargv,
			   sanson_arg_stack_t *senv)
{
  sanson_execution_t set;
    
  sanson_argv_t *tmpargv;
  sanson_argv_t *tmpenv;

  void *leon_mmap, *leon_mmap_code;

  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(sargv, NULL, -1);
  XCMP(senv, NULL, -1);

#if __DEBUG_SANSON__
  sanson_paquet_view(spt);
  view_binary_maps();
#endif

  memset(&set, '\0', sizeof(sanson_execution_t));

  sanson_debug("[+] ARGV %d strlen(%d)\n", sargv->argc, sargv->size);
  for (tmpargv = sargv->argv.tqh_first; 
       tmpargv != NULL; 
       tmpargv = tmpargv->entries.tqe_next)
    {
      sanson_debug("\t [+] %s\n",
		   tmpargv->name);
    }

  sanson_debug("[+] ENV %d strlen(%d)\n", senv->argc, senv->size);
  for (tmpenv = senv->argv.tqh_first; 
       tmpenv != NULL; 
       tmpenv = tmpenv->entries.tqe_next)
    {
      sanson_debug("\t [+] %s\n",
		   tmpenv->name);
    }


  /* Uncrypt */
  if (userland_execve_paquet_uncrypt(sst, spt) < 0)
    return -1;

  /* Make the binary */
  switch(spt->spht.type)
    {
    case SANSON_TYPE_STATIC :
      userland_execve_paquet_static(sst, spt, sargv, senv, &set);
      break;
    case SANSON_TYPE_DYN :
      userland_execve_paquet_dyn(sst, spt, sargv, senv, &set);
      break;
    default :
      return -1;
    }

  /* Prepare leon */

  sst->alloc_flags = PROT_READ|PROT_EXEC|PROT_WRITE;
  sst->alloc_map = MAP_PRIVATE|MAP_ANONYMOUS;

  leon_mmap = sanson_alloc(sst, set.vtt.lenc + LEON_SIZE);

  memcpy(leon_mmap, set.vtt.buffer, set.vtt.lenc);
  memcpy(leon_mmap+set.vtt.lenc, lleon, LEON_SIZE);

  sanson_debug("Alloc Leon @ 0x%lx strlen(%d)\n", 
	       (unsigned long)leon_mmap, 
	       set.vtt.lenc + LEON_SIZE);

  leon_mmap_code = leon_mmap + set.vtt.lenc;

  /* Clear this target's buffer of the stack */
  memset(&set.vtt, '\0', sizeof(virtual_target_t));

  switch(spt->spht.type)
    {
    case SANSON_TYPE_STATIC :
      sanson_session_free_without_paquet(sst, spt);
      break;

    case SANSON_TYPE_DYN :
      sanson_session_free(sst);
      break;
    }

  /* Jump to leon */
  /* Stack context :
     - @ targets (leon_mmap)
     - @ entry point
     - @ stack pointer
  */

  fprintf(stdout, "Jump @ 0x%lx EP @ 0x%lx ESP @ 0x%lx VT @ 0x%lx\n", 
	  (unsigned long)leon_mmap_code,
	  (unsigned long)set.entry,
	  (unsigned long)set.esp,
	  (unsigned long)leon_mmap);

  /* Ok ok, we find the holy grail ? */
  __asm__(
	  "push %0\n"
	  "push %1\n"
	  "push %2\n"
	  "jmp *%3\n"
	  :
	  : "a" (set.entry), "b" (set.esp), "c" (leon_mmap), "d" (leon_mmap_code)
	  );

  return -1;
}
