#include "sanson.h"

int userland_execve_view_dynsym(sanson_session_t *sst, sanson_paquet_t *spt, sanson_execution_t *set)
{
  int i;
  Elf32_Ehdr *hdr = (Elf32_Ehdr *) spt->binary;
  Elf32_Shdr *secstring;
  Elf32_Shdr *section_dynsym;
  Elf32_Sym *sym;
  char *buffer;

  section_dynsym = spt->binary + hdr->e_shoff+set->idxdynsym*sizeof(Elf32_Shdr);

  secstring = spt->binary + hdr->e_shoff+sizeof(Elf32_Shdr)*set->idxdynstr;
  
  buffer = spt->binary + secstring->sh_offset;

  i = 0;
  while (i < section_dynsym->sh_size)
    {
      sym = spt->binary + section_dynsym->sh_offset + i;

      if (i != 0)
	printf("%d ", i/sizeof(Elf32_Sym));
      else
	printf("%d ", i);

      printf("[?] ST_VALUE 0x%x ST_NAME 0x%x ST_SIZE %d ST_INFO %d ST_SHNDX %d => %s\n",
	     sym->st_value, sym->st_name, sym->st_size, sym->st_info, sym->st_shndx, buffer+sym->st_name);

      i += sizeof(Elf32_Sym);
    }

  return 0;
}

int userland_execve_view_reloc(sanson_session_t *sst, sanson_execution_t *set)
{
  sanson_func_reloc_t *tmp;
  Elf32_Sym *sym;
  char *buffer;

  sanson_debug("Reloc %d symbols\n", set->nbsym);
  for (tmp = set->stsfr.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next)
    {
      sym = tmp->sym;
      buffer = tmp->buffer;

      sanson_debug("Rel %15s @ Off %x Type %d st_value 0x%.8lx => ", buffer+sym->st_name, tmp->off, tmp->type, sym->st_value);
      switch(tmp->space)
	{
	case 0 :
	  sanson_debug("PROCESSUS\n");
	  break;
	case 1 :
	  sanson_debug("LIBC\n");
	  break;
	default :
	  sanson_debug("UND\n");
	  break;
	}
    }

  sanson_debug("\n");
  return 0;
}

int userland_execve_view_plt(sanson_session_t *sst, sanson_execution_t *set)
{
  sanson_func_plt_t *tmp;
  Elf32_Sym *sym;

  for (tmp = set->stsfp.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next)
    {
      sym = tmp->sym;
      sanson_debug("\t %s @ 0x%lx\n", tmp->buffer+sym->st_name, tmp->addr);
    }

  return 0;
}

int userland_execve_free_reloc(sanson_session_t *sst, sanson_execution_t *set)
{
  sanson_func_reloc_t *tmp, *next;

  for (tmp = set->stsfr.tqh_first; tmp != NULL; tmp = next)
    {
      next = tmp->entries.tqe_next;
      TAILQ_REMOVE(&set->stsfr, tmp, entries);

      sanson_free(sst, tmp, sizeof(sanson_func_reloc_t));
    }

  return 0;
}

int userland_execve_free_plt(sanson_session_t *sst, sanson_execution_t *set)
{
  sanson_func_plt_t *tmp, *next;

  for (tmp = set->stsfp.tqh_first; tmp != NULL; tmp = next)
    {
      next = tmp->entries.tqe_next;
      TAILQ_REMOVE(&set->stsfp, tmp, entries);

      sanson_free(sst, tmp, sizeof(sanson_func_plt_t));
    }

  return 0;
}

int userland_execve_free_sym(sanson_session_t *sst, sanson_execution_t *set)
{
  sanson_func_sym_t *tmp, *next;

  for (tmp = set->stsfs.tqh_first; tmp != NULL; tmp = next)
    {
      next = tmp->entries.tqe_next;
      TAILQ_REMOVE(&set->stsfs, tmp, entries);

      sanson_free(sst, tmp, sizeof(sanson_func_sym_t));
    }

  return 0;
}

Elf32_Sym *userland_execve_get_name(sanson_paquet_t *spt, 
				    sanson_execution_t *set,
				    int idxsection, int idxsecstring, int idx)
 {
  Elf32_Ehdr *hdr = (Elf32_Ehdr *) spt->binary;
  Elf32_Shdr *secstring;
  Elf32_Shdr *section;
  Elf32_Sym *sym;
  char *buffer;

  section = spt->binary + hdr->e_shoff+idxsection*sizeof(Elf32_Shdr);
  secstring = spt->binary + hdr->e_shoff+sizeof(Elf32_Shdr)*idxsecstring;
  
  buffer = spt->binary + secstring->sh_offset;

  sym = spt->binary + section->sh_offset + idx * sizeof(Elf32_Sym);

  return sym;
}

int userland_execve_feed_elf(sanson_paquet_t *spt, sanson_execution_t *set)
{
  int i;
  Elf32_Ehdr *hdr = (Elf32_Ehdr *) spt->binary;

  Elf32_Shdr *secstring;
  Elf32_Shdr *section;
  char *buffer;

  secstring =  spt->binary + hdr->e_shoff+sizeof(Elf32_Shdr)*hdr->e_shstrndx;
  buffer = spt->binary + secstring->sh_offset;

  for (i=0; i < hdr->e_shnum; i++)
    {
      section = spt->binary + hdr->e_shoff+i*sizeof(Elf32_Shdr);

      if (!strcmp(buffer+section->sh_name, ".rel.plt"))
	set->idxrelplt = i;

      if (!strcmp(buffer+section->sh_name, ".rel.dyn"))
	set->idxreldyn = i;

      if (!strcmp(buffer+section->sh_name, ".plt"))
	set->idxplt = i;

      if (!strcmp(buffer+section->sh_name, ".strtab"))
	set->idxstrtab = i;

      if (!strcmp(buffer+section->sh_name, ".symtab"))
	set->idxsymtab = i;

      if (!strcmp(buffer+section->sh_name, ".dynsym"))
	set->idxdynsym = i;

      if (!strcmp(buffer+section->sh_name, ".dynstr"))
	set->idxdynstr = i;

      if (!strcmp(buffer+section->sh_name, ".got.plt"))
	set->idxgotplt = i;

    }

  return 0;
}

int userland_execve_get_libc_symbols(sanson_session_t *sst, 
				     sanson_execution_t *set)
{
  int i, j, k;
  void *ptr;  
  Elf32_Ehdr *hdr;
  Elf32_Phdr *phdr;
  Elf32_Dyn *dyn;
  Elf32_Sym *sym_libc, *sym_proc;
  char *buffer;
  sanson_func_reloc_t *tmp;

  ptr = (unsigned long *)set->process_libc_begin;

  hdr = (Elf32_Ehdr *) ptr;

  for (i=0; i < hdr->e_phnum; i++)
    {
       phdr = ptr + hdr->e_phoff + i * hdr->e_phentsize;

      if (phdr->p_type == PT_DYNAMIC)
	{
	  for (j = 0; j < phdr->p_memsz; j+=sizeof(Elf32_Dyn))
	    {
	      dyn = ptr + phdr->p_offset + j;

	      if (dyn->d_tag == DT_STRTAB)
		{
		  //printf("STRTAB\n");
		  buffer = (void *)dyn->d_un.d_ptr;
		}

	      if (dyn->d_tag == DT_SYMTAB)
		{
		  //printf("SYMTAB %x\n", dyn->d_un.d_ptr);
		  k = 0;
		  while((void *)sym_libc != (void *)buffer)
		    {
		      sym_libc = (Elf32_Sym *)(dyn->d_un.d_ptr + k * sizeof(Elf32_Sym));
		      //printf("%d %x VALUE %x SIZE %x NAME %x BUFFER %x => %s\n", k, sym, sym->st_value, sym->st_size, sym->st_name, buffer, buffer+sym->st_name);

		      for (tmp = set->stsfr.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next)
			{
			  sym_proc = tmp->sym;
			  
			  if (sym_proc->st_value == 0)
			    {
				if (!strcmp(tmp->buffer+sym_proc->st_name, buffer+sym_libc->st_name))
				  {
				    sanson_debug("MATCHED IN LIBC %15s @ 0x%.8lx => ", 
						 buffer+sym_libc->st_name, 
						 sym_proc->st_value);
				    sym_proc->st_value = set->process_libc_begin + sym_libc->st_value;
				    sanson_debug("0x%.8lx\n", sym_proc->st_value);
				    tmp->space = 1;  
				  }
			    }
			}
		      k++;
		    }				  
		}
	    }
	}
    }

  return 0;
}

int userland_execve_make_reloc(sanson_session_t *sst, 
			       sanson_execution_t *set)
{
  int i;
  sanson_func_reloc_t *tmp;
  sanson_func_plt_t *tmp2;
  Elf32_Rel *rel;
  Elf32_Sym *sym, *sym2;
  char *buffer;

  i = 0;
  for (tmp = set->stsfr.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next)
    {
      rel = tmp->rel;
      sym = tmp->sym;
      buffer = tmp->buffer;
      
      if (sym->st_value != 0)
	{
	  sanson_debug("RELOC LOCAL FUNC %15s @ 0x%.8lx => ", buffer+sym->st_name, sym->st_value);
	  sym->st_value += (unsigned long)set->mmaptr;
	  sanson_debug("0x%.8lx\n", sym->st_value);
	  tmp->space = 0;
	  i++;
	}
      else
	{
	  for (tmp2 = set->stsfp.tqh_first; tmp2 != NULL; tmp2 = tmp2->entries.tqe_next)
	    {
	      sym2 = tmp2->sym;
	      if (!strcmp(tmp2->buffer+sym2->st_name, buffer+sym->st_name))
		{
		  sanson_debug("MATCHED IN PROCESSUS %15s @ 0x%.8lx => ", 
			       buffer+sym->st_name, 
			       sym->st_value);
		  sym->st_value = tmp2->addr;
		  sanson_debug("0x%.8lx\n", sym->st_value);
		  tmp->space = 0;
		  i++;
		  break;
		}
	    }
	}
    }

  if (i != set->nbsym)
    {
      sanson_debug("Found %d in processus, remained %d searching in libc\n", i, set->nbsym - i);
      userland_execve_get_libc_symbols(sst, set);
    }

  i = 0;
  for (tmp = set->stsfr.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next)
    {
      if (tmp->space == -1)
	{
	  sanson_debug("RELOC symb %s failed\n", buffer+sym->st_name);
	  i++;
	}
    }

  if (i > 0)
    {
      sanson_debug("Relocation failed !!!!!\n");
      return -1;
    }

  sanson_debug("Relocation successed !!!\n");

  return 0;
}

unsigned long userland_execve_get_main(sanson_session_t *sst, 
				       sanson_paquet_t *spt, 
				       sanson_execution_t *set, 
				       char *main_func)
{
  int i;
  Elf32_Ehdr *hdr = (Elf32_Ehdr *) spt->binary;
  Elf32_Shdr *secstring;
  Elf32_Shdr *section;
  Elf32_Sym *sym;
  char *buffer;

  section = spt->binary + hdr->e_shoff+set->idxsymtab*sizeof(Elf32_Shdr);
  secstring = spt->binary + hdr->e_shoff+sizeof(Elf32_Shdr)*set->idxstrtab;
  
  buffer = spt->binary + secstring->sh_offset;

  for (i = 0; i < section->sh_size; i += sizeof(Elf32_Sym))
    {
      sym = spt->binary + section->sh_offset + i;

      if (!strcmp(buffer+sym->st_name, main_func))
	{
	  return sym->st_value;
	}
    }    

  return 0;
}

int userland_execve_get_reloc(sanson_session_t *sst, sanson_execution_t *set, unsigned long val)
{
  int i;
  sanson_func_reloc_t *tmp;
  Elf32_Rel *rel;
  Elf32_Sym *sym;
  char *buffer;

  for (i = 0, tmp = set->stsfr.tqh_first; tmp != NULL; tmp = tmp->entries.tqe_next, i++)
    {
      rel = tmp->rel;
      sym = tmp->sym;
      buffer = tmp->buffer;

      if (tmp->off == val)
	  return sym->st_value;
    }

  return 0;
}

int userland_execve_get_relplt(sanson_session_t *sst, sanson_paquet_t *spt, sanson_execution_t *set)
{
  int i, j;
  Elf32_Shdr *section;
  Elf32_Shdr *secstring;
  Elf32_Ehdr *hdr;
  Elf32_Sym *sym;
  Elf32_Rel *rel;
  sanson_func_reloc_t *sfrt;
  char *buffer;

  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(set, NULL, -1);

  hdr = (Elf32_Ehdr *) spt->binary;
  section = spt->binary + hdr->e_shoff+set->idxrelplt*sizeof(Elf32_Shdr);
  secstring = spt->binary + hdr->e_shoff+sizeof(Elf32_Shdr)*set->idxdynstr;

  i = 0;
  j = 0;
  while (j < section->sh_size)
    {
      rel = spt->binary + section->sh_offset + j;
      
      buffer = spt->binary + secstring->sh_offset;

      /* Add all relocatable symbols */
      sym = userland_execve_get_name(spt, set, set->idxdynsym, set->idxdynstr, ELF32_R_SYM(rel->r_info));

      sfrt = sanson_alloc(sst, sizeof(sanson_func_reloc_t));
      sfrt->rel = rel;
      sfrt->sym = sym;
      sfrt->buffer = buffer;
      sfrt->type = 0;
      sfrt->off = 0xc + i * 4;
      sfrt->space = -1;

      set->nbsym++;
      set->nbrelplt++;

      TAILQ_INSERT_TAIL(&set->stsfr, sfrt, entries);

      j += sizeof(Elf32_Rel);
      i++;
    }

  return 0;
}

int userland_execve_get_reldyn(sanson_session_t *sst, sanson_paquet_t *spt, sanson_execution_t *set)
{
  int i, j;
  Elf32_Shdr *section;
  Elf32_Shdr *secstring;
  Elf32_Ehdr *hdr;
  Elf32_Sym *sym;
  Elf32_Rel *rel;
  sanson_func_reloc_t *sfrt;
  char *buffer;

  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(set, NULL, -1);

  hdr = (Elf32_Ehdr *) spt->binary;

  section = spt->binary + hdr->e_shoff+set->idxreldyn*sizeof(Elf32_Shdr);
  secstring = spt->binary + hdr->e_shoff+sizeof(Elf32_Shdr)*set->idxdynstr;

  i = 0;
  j = 0;
  while (j < section->sh_size)
    {
      rel = spt->binary + section->sh_offset + j;
      
      buffer = spt->binary + secstring->sh_offset;

      /* Add all relocatable symbols */
      sym = userland_execve_get_name(spt, set, set->idxdynsym, set->idxdynstr, ELF32_R_SYM(rel->r_info));

      sfrt = sanson_alloc(sst, sizeof(sanson_func_reloc_t));
      sfrt->rel = rel;
      sfrt->sym = sym;
      sfrt->buffer = buffer;
      sfrt->type = 1;
      sfrt->off = (section->sh_size/sizeof(Elf32_Rel) - i) * -4;
      sfrt->space = -1;

      set->nbsym++;
      set->nbreldyn++;

      TAILQ_INSERT_TAIL(&set->stsfr, sfrt, entries);

      j += sizeof(Elf32_Rel);
      i++;
    }

  return 0;
}

int userland_execve_get_processplt(sanson_session_t *sst, sanson_paquet_t *spt, sanson_execution_t *set)
{
  int i;
  unsigned long *ptr;
  void *nplt, *nnplt, *next;
  sanson_func_plt_t *sfpt; 

  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(set, NULL, -1);

  /* Now, recovered process' plt */
  ptr = (unsigned long *)set->process_text_begin;

  i = 0;
  while(ptr < (unsigned long *)set->process_text_end)
    {
      if ((nplt = find_pattern(ptr, 6, "\xff\x25", strlen("\xff\x25"))))
	{
	  if ((nnplt = find_pattern(nplt, 16, "\x68", strlen("\x68"))) 
	      && find_pattern(nnplt, 16, "\xe9", strlen("\xe9")))
	    {
	      if (nplt - next == 16)
		{
		  //  printf("I %d 0x%lx\n", i, (unsigned long)(nplt));
		  
		  sfpt = sanson_alloc(sst, sizeof(sanson_func_plt_t));
		  sfpt->addr = (unsigned long)(nplt);
		  sfpt->idx = i;
		  TAILQ_INSERT_TAIL(&set->stsfp, sfpt, entries);

		  i++;
		  next = nplt;
		  ptr = nplt + 16;
		}
	      else
		ptr++;
	    }
	  else
	    {
	      ptr++;
	    }
	}
      else if ((nplt = find_pattern(ptr, 6, "\xff\x35", strlen("\xff\x35"))))
	{
	  if ((nnplt = find_pattern(nplt, 14, "\xff\x25", strlen("\xff\x25"))) 
	      && find_pattern(nnplt, 14, "\x00\x00", strlen("\x00\x00")))
	    {
	      //   printf("I %d 0x%lx\n", i, (unsigned long)(nplt));
	      sfpt = sanson_alloc(sst, sizeof(sanson_func_plt_t));
	      sfpt->addr = (unsigned long)(nplt);
	      sfpt->idx = i;
	      TAILQ_INSERT_TAIL(&set->stsfp, sfpt, entries);

	      i++;
	      next = nplt;
	      
	      ptr = nplt + 14;
	    }
	  else
	      ptr++;
	}
      else
	ptr++;
    }
  return 0;
}

int userland_execve_get_processsymbs(sanson_session_t *sst, sanson_paquet_t *spt, sanson_execution_t *set)
{
  int i, j;
  sanson_func_plt_t *sfpt; 
  Elf32_Ehdr *hdr;
  Elf32_Phdr *phdr;
  Elf32_Sym *sym;
  Elf32_Dyn *dyn;
  void *base;

  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(set, NULL, -1);

  /* recovered symbol's name in the current process */
  base = (void *)set->process_text_begin;
  hdr = (Elf32_Ehdr *) base;
  
  for (i = 0; i < hdr->e_phnum; i++)
    {
      phdr = base + hdr->e_phoff + i * hdr->e_phentsize;

      if (phdr->p_type == PT_DYNAMIC)
	{
	  sanson_debug("DYNAMIC SECTION\n");

	  for (j = 0; j < phdr->p_memsz; j+=sizeof(Elf32_Dyn))
	    {
	      dyn = base + phdr->p_offset + j;
	      if (dyn->d_tag == DT_STRTAB)
		  for (sfpt = set->stsfp.tqh_first; sfpt != NULL; sfpt = sfpt->entries.tqe_next)
		      sfpt->buffer = (void *)dyn->d_un.d_ptr;

	      if (dyn->d_tag == DT_SYMTAB)
		{
		  sym = (Elf32_Sym *)(dyn->d_un.d_ptr);
		  for (sfpt = set->stsfp.tqh_first; sfpt != NULL; sfpt = sfpt->entries.tqe_next)
		    {
		      sfpt->sym = sym;
		      sym ++;
		    }
		}
	    }
	}
    }

  return 0;
}

int userland_execve_get_processrel(sanson_session_t *sst, sanson_paquet_t *spt, sanson_execution_t *set)
{
  int i, j, k;
  Elf32_Ehdr *hdr;
  Elf32_Phdr *phdr;
  Elf32_Dyn *dyn;
  void *base;
  Elf32_Rel *rel;
  sanson_func_sym_t *sfst;
  unsigned long dt_jmprel;

  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(set, NULL, -1);

  base = (void *)set->process_text_begin;
  hdr = (Elf32_Ehdr *) base;

  for (i = 0; i < hdr->e_phnum; i++)
    {
      phdr = base + hdr->e_phoff + i * hdr->e_phentsize;
      
      if (phdr->p_type == PT_DYNAMIC)
	{
	  for (j = 0; j < phdr->p_memsz; j+=sizeof(Elf32_Dyn))
	    {
	      dyn = base + phdr->p_offset + j;

	      if (dyn->d_tag == DT_JMPREL)
		dt_jmprel = (unsigned long)dyn->d_un.d_ptr;
		
	      if (dyn->d_tag == DT_REL)
		{
		  //sanson_hexdump((unsigned char *)dyn->d_un.d_ptr, 40, dyn->d_un.d_ptr);
		  
		  rel = NULL;
		  k = 0;
		  while((unsigned long)rel != dt_jmprel)
		    {
		      rel = (Elf32_Rel *)(dyn->d_un.d_ptr + k * sizeof(Elf32_Rel));
		      //printf("0x%lx 0x%lx 0x%lx\n", (unsigned long)rel, rel->r_offset, rel->r_info);
		      sfst = sanson_alloc(sst, sizeof(sanson_func_sym_t));
		      sfst->sym = NULL;
		      sfst->addr = rel->r_offset;
		      sfst->buffer = NULL;
		      
		      TAILQ_INSERT_TAIL(&set->stsfs, sfst, entries);

		      k++;
		      rel = (Elf32_Rel *)(dyn->d_un.d_ptr + k * sizeof(Elf32_Rel));
		    }
		}
	    }
	}
    }
 
 return 0;
}

int userland_execve_get_processrelsymbs(sanson_session_t *sst, sanson_paquet_t *spt, sanson_execution_t *set)
{
  int i, j, k;
  Elf32_Ehdr *hdr;
  Elf32_Phdr *phdr;
  Elf32_Dyn *dyn;
  Elf32_Sym *sym;
  char *buffer;
  void *base;
  sanson_func_sym_t *sfst;
		      
  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(set, NULL, -1);

  base = (void *)set->process_text_begin;
  hdr = (Elf32_Ehdr *) base;


  for (i = 0; i < hdr->e_phnum; i++)
    {
      phdr = base + hdr->e_phoff + i * hdr->e_phentsize;
      
      if (phdr->p_type == PT_DYNAMIC)
	{	  
	  for (j = 0; j < phdr->p_memsz; j+=sizeof(Elf32_Dyn))
	    {
	      dyn = base + phdr->p_offset + j;
	      if (dyn->d_tag == DT_STRTAB)
		buffer = (void *)dyn->d_un.d_ptr;
	      
	      if (dyn->d_tag == DT_SYMTAB)
		{
		  //printf("SYMTAB %x\n", dyn->d_un.d_ptr);

		  k = 0;
		  sym = NULL;
		  while((unsigned long)sym != (unsigned long)buffer)
		    {
		      sym = (Elf32_Sym *)(dyn->d_un.d_ptr + k * sizeof(Elf32_Sym));
		      //		      printf("%d %x VALUE %x SIZE %x NAME %x BUFFER %x => %s\n", k, sym, sym->st_value, sym->st_size, sym->st_name, buffer, buffer+sym->st_name);
		      		      
		      for (sfst = set->stsfs.tqh_first; sfst != NULL; sfst = sfst->entries.tqe_next)
			{
			  if (sfst->addr == sym->st_value)
			    {
			      printf("Link rel -> symbs to %s\n", buffer+sym->st_name);
			      sfst->sym = sym;
			      sfst->buffer = buffer;
			    }
			}
		      
		      k++;
		      sym = (Elf32_Sym *)(dyn->d_un.d_ptr + k * sizeof(Elf32_Sym));
		    }
		}
	    }
	}
    }
  
  return 0;
}

int userland_execve_patch_got(sanson_session_t *sst, 
			      sanson_paquet_t *spt, 
			      sanson_execution_t *set)
{
  int i, j, ret;
  Elf32_Ehdr *hdr;
  Elf32_Shdr *section;
  Elf32_Sym *sym;
  Elf32_Rel *rel;
  char *buffer;  
  void *gotaddr, *grepplt, *findop;
  sanson_func_sym_t *sfst;
  sanson_func_reloc_t *sfrt;
  unsigned long val, newplt;

 
  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(set, NULL, -1);

  ret = 0;
  hdr = (Elf32_Ehdr *) spt->binary;

  section = spt->binary + hdr->e_shoff+set->idxgotplt*sizeof(Elf32_Shdr);
  gotaddr = set->mmaptr + section->sh_addr;

  section = spt->binary + hdr->e_shoff+set->idxplt*sizeof(Elf32_Shdr);

  sanson_debug("GOT @ 0x%lx\n", 
	       (unsigned long)gotaddr);

  i = 0;
  /* .rel.plt */
  grepplt = spt->binary + section->sh_offset;
  while(grepplt < (spt->binary + section->sh_offset + section->sh_size))
    {
      if ((findop = find_pattern(grepplt, 6, "\xff\xa3", strlen("\xff\xa3"))))
	{
	  memcpy(&val, findop+2, sizeof(val));
	 
	  newplt = userland_execve_get_reloc(sst, set, val);
	  if (newplt == 0)
	    {
	      grepplt+=6;
	      continue;
	    }

	  sanson_debug("OFF 0x%.8lx => \t PATCH GOT .rel.plt @ 0x%.8lx with 0x%lx\n", 
		       val, 
		       gotaddr + val, 
		       newplt);
	  memcpy(gotaddr + val, &newplt, sizeof(newplt));
	  grepplt += 6;
	  i++;
	}	  
      grepplt++;
    }
    
  j = 0;
  /* .rel.dyn */
  
  for (sfrt = set->stsfr.tqh_first; sfrt != NULL; sfrt = sfrt->entries.tqe_next)
    {
      if (sfrt->type == 1)
	{
	  rel = sfrt->rel;
	  sym = sfrt->sym;
	  buffer = sfrt->buffer;

	  for (sfst = set->stsfs.tqh_first; sfst != NULL; sfst = sfst->entries.tqe_next)
	    {    
	      if (sfst->sym != NULL && sfst->buffer != NULL)
		{
	
		  if (!strcmp(sfst->buffer+sfst->sym->st_name, buffer+sym->st_name))
		    {		      
		      sanson_debug("OFF 0x%.8lx => \t PATCH GOT .rel.dyn @ 0x%.8lx with 0x%lx\n", 
				   sfrt->off, 
				   gotaddr + sfrt->off, 
				   sfst->addr);
		      
		      memcpy(gotaddr + sfrt->off, &sfst->addr, sizeof(sfst->addr));
		      j++;
		    }
		}
	    }
	}
   }
   /*
  int k = 0;
  void *truc;

  sanson_session_alloc_set(sst, SANSON_ALLOC_MMAP);

  truc = sanson_alloc(sst, set->nbreldyn * sizeof(unsigned long));
  printf("TRUC @ 0x%lx %d\n", (unsigned long) truc,  set->nbreldyn * sizeof(unsigned long));
  memset(truc, '\0', set->nbreldyn * sizeof(unsigned long));
  sanson_session_alloc_set(sst, SANSON_ALLOC_MALLOC);

  for (sfrt = set->stsfr.tqh_first; sfrt != NULL; sfrt = sfrt->entries.tqe_next)
    {
      if (sfrt->type == 1)
	{
	  rel = sfrt->rel;
	  sym = sfrt->sym;
	  buffer = sfrt->buffer;

	  unsigned long toto = (unsigned long)truc;

	  memcpy(truc + k, &sym->st_value, sizeof(sym->st_value));
	  sanson_hexdump(truc, set->nbreldyn * sizeof(unsigned long), truc);

	  sanson_debug("OFF 0x%.8lx => \t PATCH GOT .rel.dyn @ 0x%.8lx with 0x%lx\n", 
				   sfrt->off, 
				   gotaddr + sfrt->off, 
				   toto);
	  memcpy(gotaddr + sfrt->off, &toto, sizeof(toto));
	  k+=4;
	}
    }*/

  if ((i+j) == set->nbsym)
    sanson_debug("GOT totaly patched %d\n", i+j);
  else
    {
      if (i < set->nbrelplt)
	{
	  sanson_debug(".rel.plt unpatched %d\n", i);
	}
      
      if (j < set->nbreldyn)
	{
	  sanson_debug(".rel.dyn unpatched %d\n", j);
	}
    }

  return ret;
}

int userland_execve_paquet_dyn_qd(sanson_session_t *sst,
				  sanson_paquet_t *spt,
				  sanson_arg_stack_t *sargv,
				  sanson_arg_stack_t *senv,
				  sanson_execution_t *set)
{
  int i;
  Elf32_Ehdr *hdr;
  Elf32_Shdr *section;
  Elf32_Phdr *phdr;
  unsigned long tot_len;


  hdr = (Elf32_Ehdr *) spt->binary;

  sanson_session_alloc_set(sst, SANSON_ALLOC_MALLOC);

  /* Find index of sections */
  userland_execve_feed_elf(spt, set);

  sanson_debug("STRTAB %d SYMTAB %d DYNSYM %d REL_DYN %d REL_PLT %d PLT %d GOT_PLT %d\n", 
	       set->idxstrtab, 
	       set->idxsymtab, 
	       set->idxdynsym, 
	       set->idxreldyn,
	       set->idxrelplt, 
	       set->idxplt, 
	       set->idxgotplt);;

  /* Get reloc functions */
  userland_execve_get_relplt(sst, spt, set);
  /* Get external variable */
  userland_execve_get_reldyn(sst, spt, set);

#if __DEBUG_SANSON__
  userland_execve_view_reloc(sst, set);
#endif

  /* Calculate total size = PT_LOAD + got */
  tot_len = 0;
  for (i = 0; i < hdr->e_phnum; i++)
    {
      phdr = spt->binary + hdr->e_phoff + i * hdr->e_phentsize;

      if(phdr->p_type == PT_LOAD)
	  tot_len += ALIGN(phdr->p_memsz, phdr->p_align);
    }

  hdr = (Elf32_Ehdr *) spt->binary;
  section = spt->binary + hdr->e_shoff+set->idxgotplt*sizeof(Elf32_Shdr);
  tot_len += section->sh_size; /* Alloc the got ! */;

  sanson_session_alloc_set(sst, SANSON_ALLOC_MMAP);

  sanson_debug("TOT_LEN 0x%lx %d\n", tot_len, (unsigned int)tot_len);

  set->mmaptr = sanson_alloc(sst, tot_len);
  memset(set->mmaptr, '\0', tot_len);

  sanson_debug("0x%lx ---> 0x%lx\n", (unsigned long)set->mmaptr, (unsigned long)(set->mmaptr+tot_len));

  sanson_session_alloc_set(sst, SANSON_ALLOC_MALLOC);

  /* Get the base address .text of process and libc */
  sanson_ul_get_text(sst, set, &set->process_text_begin, &set->process_text_end);
  sanson_ul_get_libc(sst, set, &set->process_libc_begin, &set->process_libc_end);

  sanson_debug("TEXT @ 0x%lx ---> 0x%lx\n", set->process_text_begin, set->process_text_end);
  sanson_debug("LIBC 0x%lx ---> 0x%lx\n", set->process_libc_begin, set->process_libc_end);

  /* Find plt of the process */
  userland_execve_get_processplt(sst, spt, set);
  /* Find symbs of the process */
  userland_execve_get_processsymbs(sst, spt, set);

  /* Find rel of the process */
  userland_execve_get_processrel(sst, spt, set);
  /* Associate rel with symbs */
  userland_execve_get_processrelsymbs(sst, spt, set);

#if __DEBUG_SANSON__
  //userland_execve_view_plt(sst, set);
#endif

  /* Just do it (the relocation ) !!!*/
  userland_execve_make_reloc(sst, set);

#if __DEBUG_SANSON__
  userland_execve_view_reloc(sst, set);
#endif

  for (i = 0; i < hdr->e_phnum; i++)
    {
      phdr = spt->binary + hdr->e_phoff + i * hdr->e_phentsize;
      
      if(phdr->p_type == PT_LOAD)
	  memcpy(set->mmaptr+phdr->p_offset, 
		 (unsigned long *)(spt->binary + phdr->p_offset),  
		 phdr->p_filesz);
    }

  /* Patch the got !! */
  userland_execve_patch_got(sst, spt, set);

  return 0;
}

int userland_execve_paquet_dyn(sanson_session_t *sst,
			       sanson_paquet_t *spt,
			       sanson_arg_stack_t *sargv,
			       sanson_arg_stack_t *senv,
			       sanson_execution_t *set)
{
  target_t tt;

  XCMP(sst, NULL, -1);
  XCMP(spt, NULL, -1);
  XCMP(sargv, NULL, -1);
  XCMP(senv, NULL, -1);
  XCMP(set, NULL, -1);

  memset(&set->stsfr, '\0', sizeof(struct s_tailq_sanson_func_reloc));
  memset(&set->stsfp, '\0', sizeof(struct s_tailq_sanson_func_plt));
  memset(&set->stsfs, '\0', sizeof(struct s_tailq_sanson_func_sym));

  TAILQ_INIT(&set->stsfr);
  TAILQ_INIT(&set->stsfp);
  TAILQ_INIT(&set->stsfs);


  sanson_debug("EXECVE PAQUET DYN\n");

  build_stack_static(sst, set, sargv, senv);

 
  switch(sst->ul_exec_dyn)
    {
    case SANSON_UL_EXEC_DYN_QUICKANDDIRTY :
      userland_execve_paquet_dyn_qd(sst, spt, sargv, senv, set);
      break;
      
    case SANSON_UL_EXEC_DYN_RELOC :
      return -1;
      break;
    }

  /* Find the entry point : by default => main */
  set->entry = (unsigned long)(set->mmaptr + userland_execve_get_main(sst, spt, set, "main"));

  /* Add the last target : type dyn */
  memset(&tt, '\0', sizeof(tt));
  
  tt.action = 1;
  add_virtual_target(&set->vtt, &tt);

#if __DEBUG_SANSON__
  /* Visualisation des targets */
  view_virtual_target(&set->vtt);
#endif

  /* Free used memory */
  userland_execve_free_reloc(sst, set);
  userland_execve_free_plt(sst, set);
  userland_execve_free_sym(sst, set);

  sanson_session_alloc_set(sst, SANSON_ALLOC_MMAP);

  return 0;
}
